# Copyright 2006-2018 Davide Alberani <da@erlug.linux.it>
#                2012 Alberto Malagoli <albemala AT gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""
This module provides functions not used directly by the imdb package,
but useful for IMDbPY-based programs.
"""

# XXX: Find better names for the functions in this module.

from __future__ import absolute_import, division, print_function, unicode_literals

import sys
import difflib
import gettext
import re
from gettext import gettext as _

PY2 = sys.hexversion < 0x3000000
if PY2:
    from cgi import escape
else:
    from html import escape

# The modClearRefs can be used to strip names and titles references from
# the strings in Movie and Person objects.
from imdb import IMDb, imdbURL_character_base, imdbURL_movie_base, imdbURL_person_base
from imdb.Character import Character
from imdb.Company import Company
from imdb.linguistics import COUNTRY_LANG
from imdb.Movie import Movie
from imdb.Person import Person
from imdb.utils import _tagAttr, re_characterRef, re_nameRef, re_titleRef
from imdb.utils import TAGS_TO_MODIFY


gettext.textdomain('imdbpy')


# An URL, more or less.
_re_href = re.compile(r'(http://.+?)(?=\s|$)', re.I)
_re_hrefsub = _re_href.sub


def makeCgiPrintEncoding(encoding):
    """Make a function to pretty-print strings for the web."""
    def cgiPrint(s):
        """Encode the given string using the %s encoding, and replace
        chars outside the given charset with XML char references.""" % encoding
        s = escape(s, quote=1)
        if isinstance(s, str):
            s = s.encode(encoding, 'xmlcharrefreplace')
        return s
    return cgiPrint


# cgiPrint uses the utf8 encoding.
cgiPrint = makeCgiPrintEncoding('utf8')

# Regular expression for %(varname)s substitutions.
re_subst = re.compile(r'%\((.+?)\)s')
# Regular expression for <if condition>....</if condition> clauses.
re_conditional = re.compile(r'<if\s+(.+?)\s*>(.+?)</if\s+\1\s*>')


def makeTextNotes(replaceTxtNotes):
    """Create a function useful to handle text[::optional_note] values.
    replaceTxtNotes is a format string, which can include the following
    values: %(text)s and %(notes)s.
    Portions of the text can be conditionally excluded, if one of the
    values is absent. E.g.: <if notes>[%(notes)s]</if notes> will be replaced
    with '[notes]' if notes exists, or by an empty string otherwise.
    The returned function is suitable be passed as applyToValues argument
    of the makeObject2Txt function."""
    def _replacer(s):
        outS = replaceTxtNotes
        if not isinstance(s, str):
            return s
        ssplit = s.split('::', 1)
        text = ssplit[0]
        # Used to keep track of text and note existence.
        keysDict = {}
        if text:
            keysDict['text'] = True
        outS = outS.replace('%(text)s', text)
        if len(ssplit) == 2:
            keysDict['notes'] = True
            outS = outS.replace('%(notes)s', ssplit[1])
        else:
            outS = outS.replace('%(notes)s', '')

        def _excludeFalseConditionals(matchobj):
            # Return an empty string if the conditional is false/empty.
            if matchobj.group(1) in keysDict:
                return matchobj.group(2)
            return ''

        while re_conditional.search(outS):
            outS = re_conditional.sub(_excludeFalseConditionals, outS)
        return outS
    return _replacer


def makeObject2Txt(movieTxt=None, personTxt=None, characterTxt=None,
                   companyTxt=None, joiner=' / ',
                   applyToValues=lambda x: x, _recurse=True):
    """"Return a function useful to pretty-print Movie, Person,
    Character and Company instances.

    *movieTxt* -- how to format a Movie object.
    *personTxt* -- how to format a Person object.
    *characterTxt* -- how to format a Character object.
    *companyTxt* -- how to format a Company object.
    *joiner* -- string used to join a list of objects.
    *applyToValues* -- function to apply to values.
    *_recurse* -- if True (default) manage only the given object.
    """
    # Some useful defaults.
    if movieTxt is None:
        movieTxt = '%(long imdb title)s'
    if personTxt is None:
        personTxt = '%(long imdb name)s'
    if characterTxt is None:
        characterTxt = '%(long imdb name)s'
    if companyTxt is None:
        companyTxt = '%(long imdb name)s'

    def object2txt(obj, _limitRecursion=None):
        """Pretty-print objects."""
        # Prevent unlimited recursion.
        if _limitRecursion is None:
            _limitRecursion = 0
        elif _limitRecursion > 5:
            return ''
        _limitRecursion += 1
        if isinstance(obj, (list, tuple)):
            return joiner.join([object2txt(o, _limitRecursion=_limitRecursion)
                                for o in obj])
        elif isinstance(obj, dict):
            # XXX: not exactly nice, neither useful, I fear.
            return joiner.join(
                ['%s::%s' % (object2txt(k, _limitRecursion=_limitRecursion),
                             object2txt(v, _limitRecursion=_limitRecursion))
                 for k, v in list(obj.items())]
            )
        objData = {}
        if isinstance(obj, Movie):
            objData['movieID'] = obj.movieID
            outs = movieTxt
        elif isinstance(obj, Person):
            objData['personID'] = obj.personID
            outs = personTxt
        elif isinstance(obj, Character):
            objData['characterID'] = obj.characterID
            outs = characterTxt
        elif isinstance(obj, Company):
            objData['companyID'] = obj.companyID
            outs = companyTxt
        else:
            return obj

        def _excludeFalseConditionals(matchobj):
            # Return an empty string if the conditional is false/empty.
            condition = matchobj.group(1)
            proceed = obj.get(condition) or getattr(obj, condition, None)
            if proceed:
                return matchobj.group(2)
            else:
                return ''
        while re_conditional.search(outs):
            outs = re_conditional.sub(_excludeFalseConditionals, outs)
        for key in re_subst.findall(outs):
            value = obj.get(key) or getattr(obj, key, None)
            if not isinstance(value, str):
                if not _recurse:
                    if value:
                        value = str(value)
                if value:
                    value = object2txt(value, _limitRecursion=_limitRecursion)
            elif value:
                value = applyToValues(str(value))
            if not value:
                value = ''
            elif not isinstance(value, str):
                value = str(value)
            outs = outs.replace('%(' + key + ')s', value)
        return outs
    return object2txt


def makeModCGILinks(movieTxt, personTxt, characterTxt=None, encoding='utf8'):
    """Make a function used to pretty-print movies and persons refereces;
    movieTxt and personTxt are the strings used for the substitutions.
    movieTxt must contains %(movieID)s and %(title)s, while personTxt
    must contains %(personID)s and %(name)s and characterTxt %(characterID)s
    and %(name)s; characterTxt is optional, for backward compatibility."""
    _cgiPrint = makeCgiPrintEncoding(encoding)

    def modCGILinks(s, titlesRefs, namesRefs, characterRefs=None):
        """Substitute movies and persons references."""
        if characterRefs is None:
            characterRefs = {}

        # XXX: look ma'... more nested scopes! <g>
        def _replaceMovie(match):
            to_replace = match.group(1)
            item = titlesRefs.get(to_replace)
            if item:
                movieID = item.movieID
                to_replace = movieTxt % {
                    'movieID': movieID,
                    'title': str(_cgiPrint(to_replace), encoding, 'xmlcharrefreplace')
                }
            return to_replace

        def _replacePerson(match):
            to_replace = match.group(1)
            item = namesRefs.get(to_replace)
            if item:
                personID = item.personID
                to_replace = personTxt % {
                    'personID': personID,
                    'name': str(_cgiPrint(to_replace), encoding, 'xmlcharrefreplace')
                }
            return to_replace

        def _replaceCharacter(match):
            to_replace = match.group(1)
            if characterTxt is None:
                return to_replace
            item = characterRefs.get(to_replace)
            if item:
                characterID = item.characterID
                if characterID is None:
                    return to_replace
                to_replace = characterTxt % {
                    'characterID': characterID,
                    'name': str(_cgiPrint(to_replace), encoding, 'xmlcharrefreplace')
                }
            return to_replace
        s = s.replace('<', '&lt;').replace('>', '&gt;')
        s = _re_hrefsub(r'<a href="\1">\1</a>', s)
        s = re_titleRef.sub(_replaceMovie, s)
        s = re_nameRef.sub(_replacePerson, s)
        s = re_characterRef.sub(_replaceCharacter, s)
        return s
    modCGILinks.movieTxt = movieTxt
    modCGILinks.personTxt = personTxt
    modCGILinks.characterTxt = characterTxt
    return modCGILinks


# links to the imdb.com web site.
_movieTxt = '<a href="' + imdbURL_movie_base + 'tt%(movieID)s">%(title)s</a>'
_personTxt = '<a href="' + imdbURL_person_base + 'nm%(personID)s">%(name)s</a>'
_characterTxt = '<a href="' + imdbURL_character_base + \
                'ch%(characterID)s">%(name)s</a>'
modHtmlLinks = makeModCGILinks(movieTxt=_movieTxt, personTxt=_personTxt,
                               characterTxt=_characterTxt)
modHtmlLinksASCII = makeModCGILinks(movieTxt=_movieTxt, personTxt=_personTxt,
                                    characterTxt=_characterTxt,
                                    encoding='ascii')


def sortedSeasons(m):
    """Return a sorted list of seasons of the given series."""
    seasons = list(m.get('episodes', {}).keys())
    seasons.sort()
    return seasons


def sortedEpisodes(m, season=None):
    """Return a sorted list of episodes of the given series,
    considering only the specified season(s) (every season, if None)."""
    episodes = []
    seasons = season
    if season is None:
        seasons = sortedSeasons(m)
    else:
        if not isinstance(season, (tuple, list)):
            seasons = [season]
    for s in seasons:
        eps_indx = list(m.get('episodes', {}).get(s, {}).keys())
        eps_indx.sort()
        for e in eps_indx:
            episodes.append(m['episodes'][s][e])
    return episodes


# Idea and portions of the code courtesy of none none (dclist at gmail.com)
_re_imdbIDurl = re.compile(r'\b(nm|tt|ch|co)([0-9]{7})\b')


def get_byURL(url, info=None, args=None, kwds=None):
    """Return a Movie, Person, Character or Company object for the given URL;
    info is the info set to retrieve, args and kwds are respectively a list
    and a dictionary or arguments to initialize the data access system.
    Returns None if unable to correctly parse the url; can raise
    exceptions if unable to retrieve the data."""
    if args is None:
        args = []
    if kwds is None:
        kwds = {}
    ia = IMDb(*args, **kwds)
    match = _re_imdbIDurl.search(url)
    if not match:
        return None
    imdbtype = match.group(1)
    imdbID = match.group(2)
    if imdbtype == 'tt':
        return ia.get_movie(imdbID, info=info)
    elif imdbtype == 'nm':
        return ia.get_person(imdbID, info=info)
    elif imdbtype == 'ch':
        return ia.get_character(imdbID, info=info)
    elif imdbtype == 'co':
        return ia.get_company(imdbID, info=info)
    return None


# Idea and portions of code courtesy of Basil Shubin.
# Beware that these information are now available directly by
# the Movie/Person/Character instances.
def fullSizeCoverURL(obj):
    """Given an URL string or a Movie, Person or Character instance,
    returns an URL to the full-size version of the cover/headshot,
    or None otherwise.  This function is obsolete: the same information
    are available as keys: 'full-size cover url' and 'full-size headshot',
    respectively for movies and persons/characters."""
    return obj.get_fullsizeURL()


def keyToXML(key):
    """Return a key (the ones used to access information in Movie and
    other classes instances) converted to the style of the XML output."""
    return _tagAttr(key, '')[0]


def translateKey(key):
    """Translate a given key."""
    return _(keyToXML(key))


# Maps tags to classes.
_MAP_TOP_OBJ = {
    'person': Person,
    'movie': Movie,
    'character': Character,
    'company': Company
}

# Tags to be converted to lists.
_TAGS_TO_LIST = dict([(x[0], None) for x in list(TAGS_TO_MODIFY.values())])
_TAGS_TO_LIST.update(_MAP_TOP_OBJ)


def tagToKey(tag):
    """Return the name of the tag, taking it from the 'key' attribute,
    if present."""
    keyAttr = tag.get('key')
    if keyAttr:
        if tag.get('keytype') == 'int':
            keyAttr = int(keyAttr)
        return keyAttr
    return tag.tag


def _valueWithType(tag, tagValue):
    """Return tagValue, handling some type conversions."""
    tagType = tag.get('type')
    if tagType == 'int':
        tagValue = int(tagValue)
    elif tagType == 'float':
        tagValue = float(tagValue)
    return tagValue


# Extra tags to get (if values were not already read from title/name).
_titleTags = ('imdbindex', 'kind', 'year')
_nameTags = ('imdbindex',)
_companyTags = ('imdbindex', 'country')


def parseTags(tag, _topLevel=True, _as=None, _infoset2keys=None, _key2infoset=None):
    """Recursively parse a tree of tags."""
    # The returned object (usually a _Container subclass, but it can
    # be a string, an int, a float, a list or a dictionary).
    item = None
    if _infoset2keys is None:
        _infoset2keys = {}
    if _key2infoset is None:
        _key2infoset = {}
    name = tagToKey(tag)
    firstChild = (tag.getchildren() or [None])[0]
    tagStr = (tag.text or '').strip()
    if not tagStr and name == 'item':
        # Handles 'item' tags containing text and a 'notes' sub-tag.
        tagContent = tag.getchildren()
        if tagContent and tagContent[0].text:
            tagStr = (tagContent[0].text or '').strip()
    infoset = tag.get('infoset')
    if infoset:
        _key2infoset[name] = infoset
        _infoset2keys.setdefault(infoset, []).append(name)
    # Here we use tag.name to avoid tags like <item title="company">
    if tag.tag in _MAP_TOP_OBJ:
        # One of the subclasses of _Container.
        item = _MAP_TOP_OBJ[name]()
        itemAs = tag.get('access-system')
        if itemAs:
            if not _as:
                _as = itemAs
        else:
            itemAs = _as
        item.accessSystem = itemAs
        tagsToGet = []
        theID = tag.get('id')
        if name == 'movie':
            item.movieID = theID
            tagsToGet = _titleTags
            ttitle = tag.find('title')
            if ttitle is not None:
                item.set_title(ttitle.text)
                tag.remove(ttitle)
        else:
            if name == 'person':
                item.personID = theID
                tagsToGet = _nameTags
                theName = tag.find('long imdb canonical name')
                if not theName:
                    theName = tag.find('name')
            elif name == 'character':
                item.characterID = theID
                tagsToGet = _nameTags
                theName = tag.find('name')
            elif name == 'company':
                item.companyID = theID
                tagsToGet = _companyTags
                theName = tag.find('name')
            if theName is not None:
                item.set_name(theName.text)
                tag.remove(theName)
        for t in tagsToGet:
            if t in item.data:
                continue
            dataTag = tag.find(t)
            if dataTag is not None:
                item.data[tagToKey(dataTag)] = _valueWithType(dataTag, dataTag.text)
        notesTag = tag.find('notes')
        if notesTag is not None:
            item.notes = notesTag.text
            tag.remove(notesTag)
        episodeOf = tag.find('episode-of')
        if episodeOf is not None:
            item.data['episode of'] = parseTags(episodeOf, _topLevel=False,
                                                _as=_as, _infoset2keys=_infoset2keys,
                                                _key2infoset=_key2infoset)
            tag.remove(episodeOf)
        cRole = tag.find('current-role')
        if cRole is not None:
            cr = parseTags(cRole, _topLevel=False, _as=_as,
                           _infoset2keys=_infoset2keys, _key2infoset=_key2infoset)
            item.currentRole = cr
            tag.remove(cRole)
        # XXX: big assumption, here.  What about Movie instances used
        #      as keys in dictionaries?  What about other keys (season and
        #      episode number, for example?)
        if not _topLevel:
            # tag.extract()
            return item
        _adder = lambda key, value: item.data.update({key: value})
    elif tagStr:
        tagNotes = tag.find('notes')
        if tagNotes is not None:
            notes = (tagNotes.text or '').strip()
            if notes:
                tagStr += '::%s' % notes
        else:
            tagStr = _valueWithType(tag, tagStr)
        return tagStr
    elif firstChild is not None:
        firstChildName = tagToKey(firstChild)
        if firstChildName in _TAGS_TO_LIST:
            item = []
            _adder = lambda key, value: item.append(value)
        else:
            item = {}
            _adder = lambda key, value: item.update({key: value})
    else:
        item = {}
        _adder = lambda key, value: item.update({name: value})
    for subTag in tag.getchildren():
        subTagKey = tagToKey(subTag)
        # Exclude dinamically generated keys.
        if tag.tag in _MAP_TOP_OBJ and subTagKey in item._additional_keys():
            continue
        subItem = parseTags(subTag, _topLevel=False, _as=_as,
                            _infoset2keys=_infoset2keys, _key2infoset=_key2infoset)
        if subItem:
            _adder(subTagKey, subItem)
    if _topLevel and name in _MAP_TOP_OBJ:
        # Add information about 'info sets', but only to the top-level object.
        item.infoset2keys = _infoset2keys
        item.key2infoset = _key2infoset
        item.current_info = list(_infoset2keys.keys())
    return item


def parseXML(xml):
    """Parse a XML string, returning an appropriate object (usually an
    instance of a subclass of _Container."""
    import lxml.etree
    return parseTags(lxml.etree.fromstring(xml))


_re_akas_lang = re.compile('(?:[(])([a-zA-Z]+?)(?: title[)])')
_re_akas_country = re.compile(r'\(.*?\)')


# akasLanguages, sortAKAsBySimilarity and getAKAsInLanguage code
# copyright of Alberto Malagoli (refactoring by Davide Alberani).
def akasLanguages(movie):
    """Given a movie, return a list of tuples in (lang, AKA) format;
    lang can be None, if unable to detect."""
    lang_and_aka = []
    akas = set((movie.get('akas') or []) + (movie.get('akas from release info') or []))
    for aka in akas:
        # split aka
        aka = aka.split('::')
        # sometimes there is no countries information
        if len(aka) == 2:
            # search for something like "(... title)" where ... is a language
            language = _re_akas_lang.search(aka[1])
            if language:
                language = language.groups()[0]
            else:
                # split countries using , and keep only the first one (it's sufficient)
                country = aka[1].split(',')[0]
                # remove parenthesis
                country = _re_akas_country.sub('', country).strip()
                # given the country, get corresponding language from dictionary
                language = COUNTRY_LANG.get(country)
        else:
            language = None
        lang_and_aka.append((language, aka[0]))
    return lang_and_aka


def sortAKAsBySimilarity(movie, title, _titlesOnly=True, _preferredLang=None):
    """Return a list of movie AKAs, sorted by their similarity to
    the given title.
    If _titlesOnly is not True, similarity information are returned.
    If _preferredLang is specified, AKAs in the given language will get
    a higher score.
    The return is a list of title, or a list of tuples if _titlesOnly is False."""
    language = movie.guessLanguage()
    # estimate string distance between current title and given title
    m_title = movie['title'].lower()
    l_title = title.lower()
    scores = []
    score = difflib.SequenceMatcher(None, m_title, l_title).ratio()
    # set original title and corresponding score as the best match for given title
    scores.append((score, movie['title'], None))
    for language, aka in akasLanguages(movie):
        # estimate string distance between current title and given title
        m_title = aka.lower()
        score = difflib.SequenceMatcher(None, m_title, l_title).ratio()
        # if current language is the same as the given one, increase score
        if _preferredLang and _preferredLang == language:
            score += 1
        scores.append((score, aka, language))
    scores.sort(reverse=True)
    if _titlesOnly:
        return [x[1] for x in scores]
    return scores


def getAKAsInLanguage(movie, lang, _searchedTitle=None):
    """Return a list of AKAs of a movie, in the specified language.
    If _searchedTitle is given, the AKAs are sorted by their similarity
    to it."""
    akas = []
    for language, aka in akasLanguages(movie):
        if lang == language:
            akas.append(aka)
    if _searchedTitle:
        scores = []
        for aka in akas:
            scores.append(difflib.SequenceMatcher(None, aka.lower(),
                                                  _searchedTitle.lower()), aka)
        scores.sort(reverse=True)
        akas = [x[1] for x in scores]
    return akas


def resizeImage(image, width=None, height=None, crop=None):
    """Return resized and cropped image url."""

    regexString = r'https://m.media-amazon.com/images/\w/\w+'

    resultImage = re.findall(regexString, image)[0]
    resultImage += '@._V1_'

    if width:
        resultImage += 'SX%s_' % width
    if height:
        resultImage += 'SY%s_' % height

    if crop:
        cropVals = ','.join(crop)
        resultImage += 'CR%s_' % cropVals

    resultImage += '.jpg'

    return resultImage



# Copyright 2009-2017 Davide Alberani <da@erlug.linux.it>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""
This module provides the logging facilities used by the imdb package.
"""

from __future__ import absolute_import, division, print_function, unicode_literals

import logging


LEVELS = {
    'debug': logging.DEBUG,
    'info': logging.INFO,
    'warn': logging.WARNING,
    'warning': logging.WARNING,
    'error': logging.ERROR,
    'critical': logging.CRITICAL
}


imdbpyLogger = logging.getLogger('imdbpy')
imdbpyStreamHandler = logging.StreamHandler()
imdbpyFormatter = logging.Formatter(
    '%(asctime)s %(levelname)s [%(name)s] %(pathname)s:%(lineno)d: %(message)s'
)
imdbpyStreamHandler.setFormatter(imdbpyFormatter)
imdbpyLogger.addHandler(imdbpyStreamHandler)


def setLevel(level):
    """Set logging level for the main logger."""
    level = level.lower().strip()
    imdbpyLogger.setLevel(LEVELS.get(level, logging.NOTSET))
    imdbpyLogger.log(logging.INFO, 'set logging threshold to "%s"',
                     logging.getLevelName(imdbpyLogger.level))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
get_top_bottom_movies.py

Usage: get_top_bottom_movies

Return top and bottom 10 movies, by ratings.
"""

import sys

# Import the IMDbPY package.
try:
    import imdb
except ImportError:
    print('You need to install the IMDbPY package!')
    sys.exit(1)


if len(sys.argv) != 1:
    print('No arguments are required.')
    sys.exit(2)

i = imdb.IMDb()

top250 = i.get_top250_movies()
bottom100 = i.get_bottom100_movies()

for label, ml in [('top 10', top250[:10]), ('bottom 10', bottom100[:10])]:
    print('')
    print('%s movies' % label)
    print('rating\tvotes\ttitle')
    for movie in ml:
        outl = '%s\t%s\t%s' % (movie.get('rating'), movie.get('votes'),
                                movie['long imdb title'])
        print(outl)



def test_movie_taglines_if_single_should_be_a_list_of_phrases(ia):
    movie = ia.get_movie('0109151', info=['taglines'])  # Matrix (V)
    taglines = movie.get('taglines', [])
    assert taglines == ["If humans don't want me... why'd they create me?"]


def test_movie_taglines_if_multiple_should_be_a_list_of_phrases(ia):
    movie = ia.get_movie('0060666', info=['taglines'])  # Manos
    taglines = movie.get('taglines', [])
    assert len(taglines) == 3
    assert taglines[0] == "It's Shocking! It's Beyond Your Imagination!"


def test_movie_taglines_if_none_should_be_excluded(ia):
    movie = ia.get_movie('1863157', info=['taglines'])  # Ates Parcasi
    assert 'taglines' not in movie



import sys
from pytest import mark


def test_search_results_should_include_correct_number_of_works_by_default(ia):
    movies = ia.search_movie_advanced('matrix')
    assert len(movies) == 20


def test_search_results_should_include_correct_number_of_works(ia):
    movies = ia.search_movie_advanced('matrix', results=250)
    assert len(movies) > 220


def test_search_results_should_include_correct_number_of_works_if_asked_less_than_available(ia):
    movies = ia.search_movie_advanced('matrix', results=25)
    assert len(movies) == 25


def test_found_movies_should_have_movie_ids(ia):
    movies = ia.search_movie_advanced('matrix', results=50)
    assert all(isinstance(m.movieID, str) for m in movies)


def test_found_movies_should_have_titles(ia):
    movies = ia.search_movie_advanced('matrix', results=50)
    assert all(isinstance(m['title'], (str, unicode) if sys.version_info < (3,) else str) for m in movies)


def test_selected_movie_should_have_correct_kind(ia):
    movies = ia.search_movie_advanced('matrix', results=50)
    selected = [m for m in movies if m.movieID == '0133093'][0]
    assert selected['kind'] == 'movie'


def test_selected_video_should_have_correct_kind(ia):
    movies = ia.search_movie_advanced('matrix', results=50)
    selected = [m for m in movies if m.movieID == '0295432'][0]
    assert selected['kind'] == 'video movie'


def test_selected_tv_movie_should_have_correct_kind(ia):
    movies = ia.search_movie_advanced('Sharknado', results=250)
    selected = [m for m in movies if m.movieID == '2724064'][0]
    assert selected['kind'] == 'tv movie'


def test_selected_tv_short_should_have_correct_kind(ia):
    movies = ia.search_movie_advanced('matrix', results=50)
    selected = [m for m in movies if m.movieID == '0274085'][0]
    assert selected['kind'] == 'tv short movie'


@mark.skip('apparently we can no longer tell a series from a movie, in search results')
def test_selected_tv_series_should_have_correct_kind(ia):
    movies = ia.search_movie_advanced('matrix', results=50)
    selected = [m for m in movies if m.movieID == '0106062'][0]
    assert selected['kind'] == 'tv series'


def test_selected_ended_tv_series_should_have_correct_kind(ia):
    movies = ia.search_movie_advanced('matrix', results=50)
    selected = [m for m in movies if m.movieID == '0364888'][0]
    assert selected['kind'] == 'tv series'


def test_selected_tv_episode_should_have_correct_kind(ia):
    movies = ia.search_movie_advanced('matrix', results=250)
    selected = [m for m in movies if m.movieID == '0594932'][0]
    assert selected['kind'] == 'episode'


def test_selected_tv_special_should_have_correct_kind(ia):
    movies = ia.search_movie_advanced('matrix', results=250)
    selected = [m for m in movies if m.movieID == '1025014'][0]
    assert selected['kind'] == 'tv special'


def test_selected_video_game_should_have_correct_kind(ia):
    movies = ia.search_movie_advanced('matrix', results=250)
    selected = [m for m in movies if m.movieID == '0277828'][0]
    assert selected['kind'] == 'video game'


def test_selected_movie_should_have_correct_year(ia):
    movies = ia.search_movie_advanced('matrix', results=50)
    selected = [m for m in movies if m.movieID == '0133093'][0]
    assert selected['year'] == 1999


def test_selected_ended_tv_series_should_have_correct_series_years(ia):
    movies = ia.search_movie_advanced('matrix', results=50)
    selected = [m for m in movies if m.movieID == '0364888'][0]
    assert selected['series years'] == '2003-2004'


@mark.skip('skipped until we found another announced title')
def test_selected_unreleased_movie_should_have_correct_state(ia):
    movies = ia.search_movie_advanced('matrix', results=50)
    selected = [m for m in movies if m.movieID == '9839912'][0]
    assert selected['state'] == 'Announced'


def test_selected_movie_should_have_correct_certificate(ia):
    movies = ia.search_movie_advanced('matrix', results=50)
    selected = [m for m in movies if m.movieID == '0133093'][0]
    assert selected['certificates'] == ['R']


def test_selected_movie_should_have_correct_runtime(ia):
    movies = ia.search_movie_advanced('matrix', results=50)
    selected = [m for m in movies if m.movieID == '0133093'][0]
    assert selected['runtimes'] == ['136']


def test_selected_movie_should_have_correct_genres(ia):
    movies = ia.search_movie_advanced('matrix', results=50)
    selected = [m for m in movies if m.movieID == '0133093'][0]
    assert selected['genres'] == ['Action', 'Sci-Fi']


def test_selected_movie_should_have_correct_rating(ia):
    movies = ia.search_movie_advanced('matrix', results=50)
    selected = [m for m in movies if m.movieID == '0133093'][0]
    assert abs(selected['rating'] - 8.7) < 0.5


def test_selected_movie_should_have_correct_number_of_votes(ia):
    movies = ia.search_movie_advanced('matrix', results=50)
    selected = [m for m in movies if m.movieID == '0133093'][0]
    assert selected['votes'] >= 1513744


def test_selected_movie_should_have_correct_metascore(ia):
    movies = ia.search_movie_advanced('matrix', results=50)
    selected = [m for m in movies if m.movieID == '0133093'][0]
    assert abs(selected['metascore'] - 73) < 5


def test_selected_movie_should_have_correct_gross(ia):
    movies = ia.search_movie_advanced('matrix', results=50)
    selected = [m for m in movies if m.movieID == '0133093'][0]
    assert selected['gross'] >= 171479930


def test_selected_movie_should_have_correct_plot(ia):
    movies = ia.search_movie_advanced('matrix', results=50)
    selected = [m for m in movies if m.movieID == '0133093'][0]
    assert selected['plot'].startswith('When a beautiful stranger')


def test_selected_movie_should_have_correct_director_imdb_ids(ia):
    movies = ia.search_movie_advanced('matrix', results=50)
    selected = [m for m in movies if m.movieID == '1830851'][0]
    assert [p.personID for p in selected['directors']] == ['0649609']


def test_selected_work_should_have_correct_director_name(ia):
    movies = ia.search_movie_advanced('matrix', results=50)
    selected = [m for m in movies if m.movieID == '1830851'][0]
    assert [p['name'] for p in selected['directors']] == ['Josh Oreck']


def test_selected_work_should_have_correct_director_imdb_ids_if_multiple(ia):
    movies = ia.search_movie_advanced('matrix', results=50)
    selected = [m for m in movies if m.movieID == '0133093'][0]
    assert [p.personID for p in selected['directors']] == ['0905154', '0905152']


def test_selected_work_should_have_correct_director_names_if_multiple(ia):
    movies = ia.search_movie_advanced('matrix', results=50)
    selected = [m for m in movies if m.movieID == '0133093'][0]
    assert [p['name'] for p in selected['directors']] == ['Lana Wachowski', 'Lilly Wachowski']


def test_selected_work_should_have_correct_cast_imdb_id(ia):
    movies = ia.search_movie_advanced('matrix', results=50)
    selected = [m for m in movies if m.movieID == '1830851'][0]
    assert [p.personID for p in selected['cast']] == ['1047143']


def test_selected_work_should_have_correct_cast_name(ia):
    movies = ia.search_movie_advanced('matrix', results=50)
    selected = [m for m in movies if m.movieID == '1830851'][0]
    assert [p['name'] for p in selected['cast']] == ['Clayton Watson']


def test_selected_work_should_have_correct_cast_imdb_ids_if_multiple(ia):
    movies = ia.search_movie_advanced('matrix', results=50)
    selected = [m for m in movies if m.movieID == '0133093'][0]
    assert [p.personID for p in selected['cast']] == ['0000206', '0000401', '0005251', '0915989']


def test_selected_work_should_have_correct_cast_names_if_multiple(ia):
    movies = ia.search_movie_advanced('matrix', results=50)
    selected = [m for m in movies if m.movieID == '0133093'][0]
    assert [p['name'] for p in selected['cast']] == [
        'Keanu Reeves',
        'Laurence Fishburne',
        'Carrie-Anne Moss',
        'Hugo Weaving'
    ]


def test_selected_tv_episode_should_have_correct_title(ia):
    movies = ia.search_movie_advanced('matrix', results=250)
    selected = [m for m in movies if m.movieID == '0594933'][0]
    assert selected['title'] == "The Making of 'The Matrix'"


def test_selected_tv_episode_should_have_correct_year(ia):
    movies = ia.search_movie_advanced('matrix', results=250)
    selected = [m for m in movies if m.movieID == '0594933'][0]
    assert selected['year'] == 1999


def test_selected_tv_episode_should_have_correct_imdb_index(ia):
    movies = ia.search_movie_advanced('matrix', results=250)
    selected = [m for m in movies if m.movieID == '1072112'][0]
    assert selected['imdbIndex'] == 'I'


def test_selected_tv_episode_should_have_correct_certificate(ia):
    movies = ia.search_movie_advanced('matrix', results=250)
    selected = [m for m in movies if m.movieID == '1072112'][0]
    assert selected['certificates'] == ['TV-PG']


def test_selected_tv_episode_should_have_correct_runtime(ia):
    movies = ia.search_movie_advanced('matrix', results=250)
    selected = [m for m in movies if m.movieID == '0594933'][0]
    assert selected['runtimes'] == ['26']


def test_selected_tv_episode_should_have_correct_genres(ia):
    movies = ia.search_movie_advanced('matrix', results=250)
    selected = [m for m in movies if m.movieID == '0594933'][0]
    assert selected['genres'] == ['Documentary', 'Short']


def test_selected_tv_episode_should_have_correct_rating(ia):
    movies = ia.search_movie_advanced('matrix', results=250)
    selected = [m for m in movies if m.movieID == '0594933'][0]
    assert abs(selected['rating'] - 7.6) < 0.5


def test_selected_tv_episode_should_have_correct_number_of_votes(ia):
    movies = ia.search_movie_advanced('matrix', results=250)
    selected = [m for m in movies if m.movieID == '0594933'][0]
    assert selected['votes'] >= 14


def test_selected_tv_episode_should_have_correct_plot(ia):
    movies = ia.search_movie_advanced('matrix', results=250)
    selected = [m for m in movies if m.movieID == '10177094'][0]
    assert selected['plot'].startswith('Roberto Leoni reviews The Matrix (1999)')


def test_selected_tv_episode_should_have_correct_director_imdb_ids(ia):
    movies = ia.search_movie_advanced('matrix', results=250)
    selected = [m for m in movies if m.movieID == '0594933'][0]
    assert [p.personID for p in selected['directors']] == ['0649609']


def test_selected_tv_episode_should_have_correct_director_names(ia):
    movies = ia.search_movie_advanced('matrix', results=250)
    selected = [m for m in movies if m.movieID == '0594933'][0]
    assert [p['name'] for p in selected['directors']] == ['Josh Oreck']


def test_selected_tv_episode_should_have_correct_cast_imdb_ids(ia):
    movies = ia.search_movie_advanced('matrix', results=250)
    selected = [m for m in movies if m.movieID == '0594933'][0]
    assert [p.personID for p in selected['cast']] == ['0000401', '0300665', '0303293', '0005251']


def test_selected_tv_episode_should_have_correct_cast_names(ia):
    movies = ia.search_movie_advanced('matrix', results=250)
    selected = [m for m in movies if m.movieID == '0594933'][0]
    assert [p['name'] for p in selected['cast']] == [
        'Laurence Fishburne',
        'John Gaeta',
        "Robert 'Rock' Galotti",
        'Carrie-Anne Moss'
    ]


def test_selected_tv_episode_should_have_series(ia):
    movies = ia.search_movie_advanced('matrix', results=250)
    selected = [m for m in movies if m.movieID == '0594933'][0]
    assert selected['episode of']['kind'] == 'tv series'


def test_selected_tv_episode_should_have_correct_series_imdb_id(ia):
    movies = ia.search_movie_advanced('matrix', results=250)
    selected = [m for m in movies if m.movieID == '0594933'][0]
    assert selected['episode of'].movieID == '0318220'


def test_selected_tv_episode_should_have_correct_series_title(ia):
    movies = ia.search_movie_advanced('matrix', results=250)
    selected = [m for m in movies if m.movieID == '0594933'][0]
    assert selected['episode of']['title'] == 'HBO First Look'


def test_selected_tv_episode_should_have_correct_series_year(ia):
    movies = ia.search_movie_advanced('matrix', results=250)
    selected = [m for m in movies if m.movieID == '1072112'][0]
    assert selected['episode of']['year'] == 2001


def test_selected_tv_episode_should_have_correct_series_series_years(ia):
    movies = ia.search_movie_advanced('matrix', results=250)
    selected = [m for m in movies if m.movieID == '1072112'][0]
    assert selected['episode of']['series years'] == '2001-2012'


def test_selected_movie_should_have_cover_url(ia):
    movies = ia.search_movie_advanced('matrix', results=50)
    selected = [m for m in movies if m.movieID == '0133093'][0]
    assert selected['cover url'].endswith('.jpg')


def test_search_results_should_include_adult_titles_if_requested(ia):
    movies = ia.search_movie_advanced('castello', adult=True, results=250)
    movies_no_adult = ia.search_movie_advanced('castello', adult=False, results=250)
    assert len(movies) > len(movies_no_adult)


def test_selected_adult_movie_should_have_correct_title(ia):
    movies = ia.search_movie_advanced('matrix', adult=True, results=250)
    selected = [m for m in movies if m.movieID == '0273126'][0]
    assert selected['title'] == 'Blue Matrix'


def test_selected_adult_movie_should_have_adult_in_genres(ia):
    movies = ia.search_movie_advanced('matrix', adult=True, results=250)
    selected = [m for m in movies if m.movieID == '0273126'][0]
    assert 'Adult' in selected['genres']

@mark.skip('IMDb sorting works in misterious ways')
def test_search_results_should_be_sortable_in_alphabetical_order_default_ascending(ia):
    movies = ia.search_movie_advanced(title='matrix', sort='alpha')
    titles = [m['title'] for m in movies]
    # assert all(a <= b for a, b in zip(titles, titles[1:]))  # fails due to IMDb
    assert sum(1 if a > b else 0 for a, b in zip(titles, titles[1:])) <= 1

@mark.skip('IMDb sorting works in misterious ways')
def test_search_results_should_be_sortable_in_alphabetical_order_ascending(ia):
    movies = ia.search_movie_advanced(title='matrix', sort='alpha', sort_dir='asc')
    titles = [m['title'] for m in movies]
    # assert all(a <= b for a, b in zip(titles, titles[1:]))  # fails due to IMDb
    assert sum(1 if a > b else 0 for a, b in zip(titles, titles[1:])) <= 1


def test_search_results_should_be_sortable_in_alphabetical_order_descending(ia):
    movies = ia.search_movie_advanced(title='matrix', sort='alpha', sort_dir='desc')
    titles = [m['title'] for m in movies]
    assert all(a >= b for a, b in zip(titles, titles[1:]))


def test_search_results_should_be_sortable_in_rating_order_default_descending(ia):
    movies = ia.search_movie_advanced(title='matrix', sort='user_rating')
    ratings = [m.get('rating', 0) for m in movies]
    assert all(a >= b for a, b in zip(ratings, ratings[1:]))


def test_search_results_should_be_sortable_in_rating_order_ascending(ia):
    movies = ia.search_movie_advanced(title='matrix', sort='user_rating', sort_dir='asc')
    ratings = [m.get('rating', float('inf')) for m in movies]
    assert all(a <= b for a, b in zip(ratings, ratings[1:]))


def test_search_results_should_be_sortable_in_rating_order_descending(ia):
    movies = ia.search_movie_advanced(title='matrix', sort='user_rating', sort_dir='desc')
    ratings = [m.get('rating', 0) for m in movies]
    assert all(a >= b for a, b in zip(ratings, ratings[1:]))


def test_search_results_should_be_sortable_in_votes_order_default_ascending(ia):
    movies = ia.search_movie_advanced(title='matrix', sort='num_votes')
    votes = [m.get('votes', float('inf')) for m in movies]
    assert all(a <= b for a, b in zip(votes, votes[1:]))


def test_search_results_should_be_sortable_in_votes_order_ascending(ia):
    movies = ia.search_movie_advanced(title='matrix', sort='num_votes', sort_dir='asc')
    votes = [m.get('votes', float('inf')) for m in movies]
    assert all(a <= b for a, b in zip(votes, votes[1:]))


def test_search_results_should_be_sortable_in_votes_order_descending(ia):
    movies = ia.search_movie_advanced(title='matrix', sort='num_votes', sort_dir='desc')
    votes = [m.get('votes', 0) for m in movies]
    assert all(a >= b for a, b in zip(votes, votes[1:]))


def test_search_results_should_be_sortable_in_gross_order_default_ascending(ia):
    movies = ia.search_movie_advanced(title='matrix', sort='boxoffice_gross_us')
    grosses = [m.get('gross', float('inf')) for m in movies]
    assert all(a <= b for a, b in zip(grosses, grosses[1:]))


def test_search_results_should_be_sortable_in_gross_order_ascending(ia):
    movies = ia.search_movie_advanced(title='matrix', sort='boxoffice_gross_us', sort_dir='asc')
    grosses = [m.get('gross', float('inf')) for m in movies]
    assert all(a <= b for a, b in zip(grosses, grosses[1:]))


def test_search_results_should_be_sortable_in_gross_order_descending(ia):
    movies = ia.search_movie_advanced(title='matrix', sort='boxoffice_gross_us', sort_dir='desc')
    grosses = [m.get('gross', 0) for m in movies]
    assert all(a >= b for a, b in zip(grosses, grosses[1:]))


def test_search_results_should_be_sortable_in_runtime_order_default_ascending(ia):
    movies = ia.search_movie_advanced(title='matrix', sort='runtime')
    runtimes = [m.get('runtime', float('inf')) for m in movies]
    assert all(a <= b for a, b in zip(runtimes, runtimes[1:]))


def test_search_results_should_be_sortable_in_runtime_order_ascending(ia):
    movies = ia.search_movie_advanced(title='matrix', sort='runtime', sort_dir='asc')
    runtimes = [int(m.get('runtimes', [float('inf')])[0]) for m in movies]
    assert all(a <= b for a, b in zip(runtimes, runtimes[1:]))


def test_search_results_should_be_sortable_in_runtime_order_descending(ia):
    movies = ia.search_movie_advanced(title='matrix', sort='runtime', sort_dir='desc')
    runtimes = [int(m.get('runtimes', [float('inf')])[0]) for m in movies]
    assert all(a >= b for a, b in zip(runtimes, runtimes[1:]))


def test_search_results_should_be_sortable_in_year_order_default_ascending(ia):
    movies = ia.search_movie_advanced(title='matrix', sort='year')
    years = [m.get('year', float('inf')) for m in movies]
    assert all(a <= b for a, b in zip(years, years[1:]))


def test_search_results_should_be_sortable_in_year_order_ascending(ia):
    movies = ia.search_movie_advanced(title='matrix', sort='year', sort_dir='asc')
    years = [m.get('year', float('inf')) for m in movies]
    assert all(a <= b for a, b in zip(years, years[1:]))


# def test_search_results_should_be_sortable_in_year_order_descending(ia):
#     movies = ia.search_movie_advanced(title='matrix', sort='year', sort_dir='desc')
#     years = [m.get('year', float('inf')) for m in movies]
#     assert all(a >= b for a, b in zip(years, years[1:]))



# Copyright 2009-2018 Davide Alberani <da@erlug.linux.it>
#                2018 H. Turgut Uyar <uyar@tekir.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""
This module provides the classes (and the instances) that are used to parse
the results of a search for a given keyword.

For example, when searching for the keyword "alabama", the parsed page
would be:

http://www.imdb.com/find?q=alabama&s=kw
"""

from __future__ import absolute_import, division, print_function, unicode_literals

from imdb.utils import analyze_title

from .piculet import Path, Rule, Rules, reducers
from .searchMovieParser import DOMHTMLSearchMovieParser
from .utils import analyze_imdbid


class DOMHTMLSearchKeywordParser(DOMHTMLSearchMovieParser):
    """A parser for the keyword search page."""

    rules = [
        Rule(
            key='data',
            extractor=Path(
                foreach='//td[@class="result_text"]',
                path='./a/text()'
            )
        )
    ]


def custom_analyze_title4kwd(title, yearNote, outline):
    """Return a dictionary with the needed info."""
    title = title.strip()
    if not title:
        return {}
    if yearNote:
        yearNote = '%s)' % yearNote.split(' ')[0]
        title = title + ' ' + yearNote
    retDict = analyze_title(title)
    if outline:
        retDict['plot outline'] = outline
    return retDict


class DOMHTMLSearchMovieKeywordParser(DOMHTMLSearchMovieParser):
    """A parser for the movie search by keyword page."""

    rules = [
        Rule(
            key='data',
            extractor=Rules(
                foreach='//h3[@class="lister-item-header"]',
                rules=[
                    Rule(
                        key='link',
                        extractor=Path('./a/@href', reduce=reducers.first)
                    ),
                    Rule(
                        key='info',
                        extractor=Path('./a//text()')
                    ),
                    Rule(
                        key='ynote',
                        extractor=Path('./span[@class="lister-item-year text-muted unbold"]/text()')
                    ),
                    Rule(
                        key='outline',
                        extractor=Path('./span[@class="outline"]//text()')
                    )
                ],
                transform=lambda x: (
                    analyze_imdbid(x.get('link')),
                    custom_analyze_title4kwd(
                        x.get('info', ''),
                        x.get('ynote', ''),
                        x.get('outline', '')
                    )
                )
            )
        )
    ]

    def preprocess_string(self, html_string):
        return html_string.replace(' + >', '>')


_OBJECTS = {
    'search_keyword_parser': ((DOMHTMLSearchKeywordParser,), {'kind': 'keyword'}),
    'search_moviekeyword_parser': ((DOMHTMLSearchMovieKeywordParser,), None)
}



def test_search_movie_if_single_should_list_one_movie(ia):
    movies = ia.search_movie('od instituta do proizvodnje')
    assert len(movies) == 1
    assert movies[0].movieID == '0483758'
    assert movies[0]['kind'] == 'short'
    assert movies[0]['title'] == 'Od instituta do proizvodnje'
    assert movies[0]['year'] == 1971


def test_search_movie_should_list_default_number_of_movies(ia):
    movies = ia.search_movie('movie')
    assert len(movies) == 20


def test_search_movie_limited_should_list_requested_number_of_movies(ia):
    movies = ia.search_movie('ace in the hole', results=98)
    assert len(movies) == 98


def test_search_movie_unlimited_should_list_correct_number_of_movies(ia):
    movies = ia.search_movie('ace in the hole', results=500)
    assert 185 <= len(movies) <= 200


def test_search_movie_if_too_many_result_should_list_upper_limit_of_movies(ia):
    movies = ia.search_movie('matrix', results=500)
    assert len(movies) == 200


def test_search_movie_if_none_should_be_empty(ia):
    movies = ia.search_movie('%e4%82%a2', results=500)
    assert movies == []


def test_search_movie_entries_should_include_movie_id(ia):
    movies = ia.search_movie('matrix')
    assert movies[0].movieID == '0133093'


def test_search_movie_entries_should_include_movie_title(ia):
    movies = ia.search_movie('matrix')
    assert movies[0]['title'] == 'The Matrix'


def test_search_movie_entries_should_include_cover_url_if_available(ia):
    movies = ia.search_movie('matrix')
    assert 'cover url' in movies[0]


def test_search_movie_entries_should_include_movie_kind(ia):
    movies = ia.search_movie('matrix')
    assert movies[0]['kind'] == 'movie'


def test_search_movie_entries_should_include_movie_kind_if_other_than_movie(ia):
    movies = ia.search_movie('matrix')
    tv_series = [m for m in movies if m.movieID == '0106062']
    assert len(tv_series) == 1
    assert tv_series[0]['kind'] == 'tv series'


def test_search_movie_entries_should_include_movie_year(ia):
    movies = ia.search_movie('matrix')
    assert movies[0]['year'] == 1999


def test_search_movie_entries_should_include_imdb_index(ia):
    movies = ia.search_movie('blink')
    movie_with_index = [m for m in movies if m.movieID == '6544524']
    assert len(movie_with_index) == 1
    assert movie_with_index[0]['imdbIndex'] == 'IV'


def test_search_movie_entries_missing_imdb_index_should_be_excluded(ia):
    movies = ia.search_movie('matrix')
    assert 'imdbIndex' not in movies[0]


def test_search_movie_entries_should_include_akas(ia):
    movies = ia.search_movie('Una calibro 20 per lo specialista')
    movie_with_aka = [m for m in movies if m.movieID == '0072288']
    assert len(movie_with_aka) == 1
    assert movie_with_aka[0]['akas'] == ['Una calibro 20 per lo specialista']


def test_search_movie_entries_missing_akas_should_be_excluded(ia):
    movies = ia.search_movie('matrix')
    assert 'akas' not in movies[0]


def test_search_movie_episodes_should_include_season_and_number(ia):
    movies = ia.search_movie('swarley')  # How I Met Your Mother S02E07
    movie_with_season_and_episode = [m for m in movies if m.movieID == '0875360']
    assert len(movie_with_season_and_episode) == 1
    assert movie_with_season_and_episode[0]['season'] == 2
    assert movie_with_season_and_episode[0]['episode'] == 7



def test_movie_votes_should_be_divided_into_10_slots(ia):
    movie = ia.get_movie('0133093', info=['vote details'])  # Matrix
    votes = movie.get('number of votes', [])
    assert len(votes) == 10


def test_movie_votes_should_be_integers(ia):
    movie = ia.get_movie('0133093', info=['vote details'])  # Matrix
    votes = movie.get('number of votes', [])
    for vote in votes:
        assert isinstance(vote, int)


def test_movie_votes_median_should_be_an_integer(ia):
    movie = ia.get_movie('0133093', info=['vote details'])  # Matrix
    median = movie.get('median')
    assert median == 9


def test_movie_votes_mean_should_be_numeric(ia):
    movie = ia.get_movie('0133093', info=['vote details'])  # Matrix
    mean = movie.get('arithmetic mean')
    assert 8.5 <= mean <= 9


def test_movie_demographics_should_be_divided_into_multiple_categories(ia):
    movie = ia.get_movie('0133093', info=['vote details'])  # Matrix
    demographics = movie['demographics']
    assert len(demographics) >= 18


def test_movie_demographics_votes_should_be_integers(ia):
    movie = ia.get_movie('0133093', info=['vote details'])  # Matrix
    top1000 = movie['demographics']['top 1000 voters']
    assert 890 <= top1000['votes'] <= 1000


def test_movie_demographics_rating_should_be_numeric(ia):
    movie = ia.get_movie('0133093', info=['vote details'])  # Matrix
    top1000 = movie['demographics']['top 1000 voters']
    assert 8 <= top1000['rating'] <= 8.5



# Copyright 2017 H. Turgut Uyar <uyar@tekir.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""
This module provides the command line interface for IMDbPY.
"""

from __future__ import absolute_import, division, print_function, unicode_literals

import sys
from argparse import ArgumentParser

from imdb import VERSION, IMDb


DEFAULT_RESULT_SIZE = 20


def list_results(items, type_, n=None):
    field = 'title' if type_ == 'movie' else 'name'
    print('  # IMDb id %s' % field)
    print('=== ======= %s' % ('=' * len(field),))
    for i, item in enumerate(items[:n]):
        print('%(index)3d %(imdb_id)7s %(title)s' % {
            'index': i + 1,
            'imdb_id': getattr(item, type_ + 'ID'),
            'title': item['long imdb ' + field]
        })


def search_item(args):
    connection = IMDb()
    n = args.n if args.n is not None else DEFAULT_RESULT_SIZE
    if args.type == 'keyword':
        items = connection.search_keyword(args.key)
        if args.first:
            items = connection.get_keyword(items[0])
            list_results(items, type_='movie', n=n)
        else:
            print('  # keyword')
            print('=== =======')
            for i, keyword in enumerate(items[:n]):
                print('%(index)3d %(kw)s' % {'index': i + 1, 'kw': keyword})
    else:
        if args.type == 'movie':
            items = connection.search_movie(args.key)
        elif args.type == 'person':
            items = connection.search_person(args.key)
        elif args.type == 'character':
            items = connection.search_character(args.key)
        elif args.type == 'company':
            items = connection.search_company(args.key)

        if args.first:
            connection.update(items[0])
            print(items[0].summary())
        else:
            list_results(items, type_=args.type, n=args.n)


def get_item(args):
    connection = IMDb()
    if args.type == 'keyword':
        n = args.n if args.n is not None else DEFAULT_RESULT_SIZE
        items = connection.get_keyword(args.key, results=n)
        list_results(items, type_='movie')
    else:
        if args.type == 'movie':
            item = connection.get_movie(args.key)
        elif args.type == 'person':
            item = connection.get_person(args.key)
        elif args.type == 'character':
            item = connection.get_character(args.key)
        elif args.type == 'company':
            item = connection.get_company(args.key)
        print(item.summary())


def list_ranking(items, n=None):
    print('  # rating   votes IMDb id title')
    print('=== ====== ======= ======= =====')
    n = n if n is not None else DEFAULT_RESULT_SIZE
    for i, movie in enumerate(items[:n]):
        print('%(index)3d    %(rating)s %(votes)7s %(imdb_id)7s %(title)s' % {
            'index': i + 1,
            'rating': movie.get('rating'),
            'votes': movie.get('votes'),
            'imdb_id': movie.movieID,
            'title': movie.get('long imdb title')
        })


def get_top_movies(args):
    connection = IMDb()
    items = connection.get_top250_movies()
    if args.first:
        connection.update(items[0])
        print(items[0].summary())
    else:
        list_ranking(items, n=args.n)


def get_bottom_movies(args):
    connection = IMDb()
    items = connection.get_bottom100_movies()
    if args.first:
        connection.update(items[0])
        print(items[0].summary())
    else:
        list_ranking(items, n=args.n)


def make_parser(prog):
    parser = ArgumentParser(prog)
    parser.add_argument('--version', action='version', version='%(prog)s ' + VERSION)

    command_parsers = parser.add_subparsers(metavar='command', dest='command')
    command_parsers.required = True

    command_search_parser = command_parsers.add_parser('search', help='search for items')
    command_search_parser.add_argument('type', help='type of item to search for',
                                       choices=['movie', 'person', 'character', 'company', 'keyword'])
    command_search_parser.add_argument('key', help='title or name of item to search for')
    command_search_parser.add_argument('-n', type=int, help='number of items to list')
    command_search_parser.add_argument('--first', action='store_true', help='display only the first result')
    command_search_parser.set_defaults(func=search_item)

    command_get_parser = command_parsers.add_parser('get', help='retrieve information about an item')
    command_get_parser.add_argument('type', help='type of item to retrieve',
                                    choices=['movie', 'person', 'character', 'company', 'keyword'])
    command_get_parser.add_argument('key', help='IMDb id (or keyword name) of item to retrieve')
    command_get_parser.add_argument('-n', type=int, help='number of movies to list (only for keywords)')
    command_get_parser.set_defaults(func=get_item)

    command_top_parser = command_parsers.add_parser('top', help='get top ranked movies')
    command_top_parser.add_argument('-n', type=int, help='number of movies to list')
    command_top_parser.add_argument('--first', action='store_true', help='display only the first result')
    command_top_parser.set_defaults(func=get_top_movies)

    command_bottom_parser = command_parsers.add_parser('bottom', help='get bottom ranked movies')
    command_bottom_parser.add_argument('-n', type=int, help='number of movies to list')
    command_bottom_parser.add_argument('--first', action='store_true', help='display only the first result')
    command_bottom_parser.set_defaults(func=get_bottom_movies)

    return parser


def main(argv=None):
    argv = argv if argv is not None else sys.argv
    parser = make_parser(prog='imdbpy')
    arguments = parser.parse_args(argv[1:])
    arguments.func(arguments)


if __name__ == '__main__':
    main()



def test_person_other_works_should_contain_correct_number_of_works(ia):
    person = ia.get_person('0000206', info=['other works'])     # Keanu Reeves
    other_works = person.get('other works', [])
    assert len(other_works) == 42


def test_person_other_works_should_contain_correct_work(ia):
    person = ia.get_person('0000206', info=['other works'])     # Keanu Reeves
    other_works = person.get('other works', [])
    assert other_works[0].startswith('(1995) Stage: Appeared')


def test_person_other_works_if_none_should_be_excluded(ia):
    person = ia.get_person('0330139', info=['other works'])     # Deni Gordon
    assert 'other works' not in person



# Copyright 2004-2018 Davide Alberani <da@erlug.linux.it>
#           2008-2018 H. Turgut Uyar <uyar@tekir.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""
This module provides the classes (and the instances) that are used to parse
the results of a search for a given title.

For example, when searching for the title "the passion", the parsed page
would be:

http://www.imdb.com/find?q=the+passion&s=tt
"""

from __future__ import absolute_import, division, print_function, unicode_literals

from imdb.utils import analyze_title

from .piculet import Path, Rule, Rules, reducers
from .utils import DOMParserBase, analyze_imdbid


class DOMHTMLSearchMovieParser(DOMParserBase):
    """A parser for the title search page."""

    rules = [
        Rule(
            key='data',
            extractor=Rules(
                foreach='//td[@class="result_text"]',
                rules=[
                    Rule(
                        key='link',
                        extractor=Path('./a/@href', reduce=reducers.first)
                    ),
                    Rule(
                        key='info',
                        extractor=Path('.//text()')
                    ),
                    Rule(
                        key='akas',
                        extractor=Path(foreach='./i', path='./text()')
                    ),
                    Rule(
                        key='cover url',
                        extractor=Path('../td[@class="primary_photo"]/a/img/@src')
                    )
                ],
                transform=lambda x: (
                    analyze_imdbid(x.get('link')),
                    analyze_title(x.get('info', '')),
                    x.get('akas'),
                    x.get('cover url')
                )
            )
        )
    ]

    def _init(self):
        self.url = ''
        self.img_type = 'cover url'

    def _reset(self):
        self.url = ''

    def postprocess_data(self, data):
        if 'data' not in data:
            data['data'] = []
        results = getattr(self, 'results', None)
        if results is not None:
            data['data'][:] = data['data'][:results]
        # Horrible hack to support AKAs.
            data['data'] = [x for x in data['data'] if x[0] and x[1]]
        if data and data['data'] and len(data['data'][0]) == 4 and isinstance(data['data'][0], tuple):
            for idx, datum in enumerate(data['data']):
                if not isinstance(datum, tuple):
                    continue
                if not datum[0] and datum[1]:
                    continue
                if datum[2] is not None:
                    akas = [aka[1:-1] for aka in datum[2]]  # remove the quotes
                    datum[1]['akas'] = akas
                if datum[3] is not None:
                    datum[1][self.img_type] = datum[3]
                data['data'][idx] = (datum[0], datum[1])
        return data

    def add_refs(self, data):
        return data


_OBJECTS = {
    'search_movie_parser': ((DOMHTMLSearchMovieParser,), None)
}



# -*- coding: utf-8 -*-

#  Copyright 2009-2017 Davide Alberani <da@erlug.linux.it>
#           2012 Alberto Malagoli <albemala AT gmail.com>
#           2009 H. Turgut Uyar <uyar@tekir.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""
This module provides functions and data to handle languages and articles
(in various languages) at the beginning of movie titles in a smart way.
"""

from __future__ import absolute_import, division, print_function, unicode_literals

# List of generic articles used when the language of the title is unknown (or
# we don't have information about articles in that language).
# XXX: Managing titles in a lot of different languages, a function to recognize
# an initial article can't be perfect; sometimes we'll stumble upon a short
# word that is an article in some language, but it's not in another; in these
# situations we have to choose if we want to interpret this little word
# as an article or not (remember that we don't know what the original language
# of the title was).
# Example: 'en' is (I suppose) an article in Some Language.  Unfortunately it
# seems also to be a preposition in other languages (French?).
# Running a script over the whole list of titles (and aliases), I've found
# that 'en' is used as an article only 376 times, and as another thing 594
# times, so I've decided to _always_ consider 'en' as a non article.
#
# Here is a list of words that are _never_ considered as articles, complete
# with the cound of times they are used in a way or another:
# 'en' (376 vs 594), 'to' (399 vs 727), 'as' (198 vs 276), 'et' (79 vs 99),
# 'des' (75 vs 150), 'al' (78 vs 304), 'ye' (14 vs 70),
# 'da' (23 vs 298), "'n" (8 vs 12)
#
# I've left in the list 'i' (1939 vs 2151) and 'uno' (52 vs 56)
# I'm not sure what '-al' is, and so I've left it out...
#
# Generic list of articles in unicode:
GENERIC_ARTICLES = (
    'the', 'la', 'a', 'die', 'der', 'le', 'el', "l'", 'il', 'das', 'les', 'i', 'o', 'ein',
    'un', 'de', 'los', 'an', 'una', 'las', 'eine', 'den', 'het', 'gli', 'lo', 'os', 'ang',
    'oi', 'az', 'een', 'ha-', 'det', 'ta', 'al-', 'mga', "un'", 'uno', 'ett', 'dem', 'egy',
    'els', 'eines', 'Ï', 'Ç', 'Ôï', 'Ïé'
)


# Lists of articles separated by language.  If possible, the list should
# be sorted by frequency (not very important, but...)
# If you want to add a list of articles for another language, mail it
# it at imdbpy-devel@lists.sourceforge.net
LANG_ARTICLES = {
    'English': ('the', 'a', 'an'),
    'Italian': ('la', 'le', "l'", 'il', 'i', 'un', 'una', 'gli', 'lo', "un'", 'uno'),
    'Spanish': (
        'la', 'lo', 'el', 'las', 'un', 'los', 'una', 'al', 'del', 'unos', 'unas', 'uno'
    ),
    'French': ('le', "l'", 'la', 'les', 'un', 'une', 'des', 'au', 'du', 'à la', 'de la', 'aux'),
    'Portuguese': ('a', 'as', 'o', 'os', 'um', 'uns', 'uma', 'umas'),
    'Turkish': ()   # Some languages doesn't have articles.
}
LANG_ARTICLESget = LANG_ARTICLES.get


# Maps a language to countries where it is the main language.
# If you want to add an entry for another language or country, mail it at
# imdbpy-devel@lists.sourceforge.net .
LANG_COUNTRIES = {
    'English': (
        'Canada', 'Swaziland', 'Ghana', 'St. Lucia', 'Liberia', 'Jamaica', 'Bahamas',
        'New Zealand', 'Lesotho', 'Kenya', 'Solomon Islands', 'United States', 'South Africa',
        'St. Vincent and the Grenadines', 'Fiji', 'UK', 'Nigeria', 'Australia', 'USA',
        'St. Kitts and Nevis', 'Belize', 'Sierra Leone', 'Gambia', 'Namibia', 'Micronesia',
        'Kiribati', 'Grenada', 'Antigua and Barbuda', 'Barbados', 'Malta', 'Zimbabwe',
        'Ireland', 'Uganda', 'Trinidad and Tobago', 'South Sudan', 'Guyana', 'Botswana',
        'United Kingdom', 'Zambia'
    ),
    'Italian': ('Italy', 'San Marino', 'Vatican City'),
    'Spanish': (
        'Spain', 'Mexico', 'Argentina', 'Bolivia', 'Guatemala', 'Uruguay', 'Peru', 'Cuba',
        'Dominican Republic', 'Panama', 'Costa Rica', 'Ecuador', 'El Salvador', 'Chile',
        'Equatorial Guinea', 'Spain', 'Colombia', 'Nicaragua', 'Venezuela', 'Honduras',
        'Paraguay'
    ),
    'French': (
        'Cameroon', 'Burkina Faso', 'Dominica', 'Gabon', 'Monaco', 'France', "Cote d'Ivoire",
        'Benin', 'Togo', 'Central African Republic', 'Mali', 'Niger', 'Congo, Republic of',
        'Guinea', 'Congo, Democratic Republic of the', 'Luxembourg', 'Haiti', 'Chad',
        'Burundi', 'Madagascar', 'Comoros', 'Senegal'
    ),
    'Portuguese': (
        'Portugal', 'Brazil', 'Sao Tome and Principe', 'Cape Verde', 'Angola', 'Mozambique',
        'Guinea-Bissau'
    ),
    'German': (
        'Liechtenstein', 'Austria', 'West Germany', 'Switzerland', 'East Germany', 'Germany'
    ),
    'Arabic': (
        'Saudi Arabia', 'Kuwait', 'Jordan', 'Oman', 'Yemen', 'United Arab Emirates',
        'Mauritania', 'Lebanon', 'Bahrain', 'Libya', 'Palestinian State (proposed)', 'Qatar',
        'Algeria', 'Morocco', 'Iraq', 'Egypt', 'Djibouti', 'Sudan', 'Syria', 'Tunisia'
    ),
    'Turkish': ('Turkey', 'Azerbaijan'),
    'Swahili': ('Tanzania',),
    'Swedish': ('Sweden',),
    'Icelandic': ('Iceland',),
    'Estonian': ('Estonia',),
    'Romanian': ('Romania',),
    'Samoan': ('Samoa',),
    'Slovenian': ('Slovenia',),
    'Tok Pisin': ('Papua New Guinea',),
    'Palauan': ('Palau',),
    'Macedonian': ('Macedonia',),
    'Hindi': ('India',),
    'Dutch': ('Netherlands', 'Belgium', 'Suriname'),
    'Marshallese': ('Marshall Islands',),
    'Korean': ('Korea, North', 'Korea, South', 'North Korea', 'South Korea'),
    'Vietnamese': ('Vietnam',),
    'Danish': ('Denmark',),
    'Khmer': ('Cambodia',),
    'Lao': ('Laos',),
    'Somali': ('Somalia',),
    'Filipino': ('Philippines',),
    'Hungarian': ('Hungary',),
    'Ukrainian': ('Ukraine',),
    'Bosnian': ('Bosnia and Herzegovina',),
    'Georgian': ('Georgia',),
    'Lithuanian': ('Lithuania',),
    'Malay': ('Brunei',),
    'Tetum': ('East Timor',),
    'Norwegian': ('Norway',),
    'Armenian': ('Armenia',),
    'Russian': ('Russia',),
    'Slovak': ('Slovakia',),
    'Thai': ('Thailand',),
    'Croatian': ('Croatia',),
    'Turkmen': ('Turkmenistan',),
    'Nepali': ('Nepal',),
    'Finnish': ('Finland',),
    'Uzbek': ('Uzbekistan',),
    'Albanian': ('Albania', 'Kosovo'),
    'Hebrew': ('Israel',),
    'Bulgarian': ('Bulgaria',),
    'Greek': ('Cyprus', 'Greece'),
    'Burmese': ('Myanmar',),
    'Latvian': ('Latvia',),
    'Serbian': ('Serbia',),
    'Afar': ('Eritrea',),
    'Catalan': ('Andorra',),
    'Chinese': ('China', 'Taiwan'),
    'Czech': ('Czech Republic', 'Czechoslovakia'),
    'Bislama': ('Vanuatu',),
    'Japanese': ('Japan',),
    'Kinyarwanda': ('Rwanda',),
    'Amharic': ('Ethiopia',),
    'Persian': ('Afghanistan', 'Iran'),
    'Tajik': ('Tajikistan',),
    'Mongolian': ('Mongolia',),
    'Dzongkha': ('Bhutan',),
    'Urdu': ('Pakistan',),
    'Polish': ('Poland',),
    'Sinhala': ('Sri Lanka',),
}

# Maps countries to their main language.
COUNTRY_LANG = {}
for lang in LANG_COUNTRIES:
    for country in LANG_COUNTRIES[lang]:
        COUNTRY_LANG[country] = lang


def toUTF8(articles):
    """Convert a list of unicode articles to utf-8 encoded strings."""
    return tuple([art.encode('utf8') for art in articles])


def toDicts(articles):
    """Given a list of unicode encoded articles, build two dictionary (one
    utf-8 encoded and another one with unicode keys) for faster matches."""
    utf8Articles = toUTF8(articles)
    return dict([(x, x) for x in utf8Articles]), dict([(x, x) for x in articles])


def addTrailingSpace(articles):
    """From the given list of unicode articles, return two
    lists (one utf-8 encoded and another one in unicode) where a space
    is added at the end - if the last char is not ' or -."""
    _spArticles = []
    _spUnicodeArticles = []
    for article in articles:
        if article[-1] not in ("'", '-'):
            article += ' '
        _spArticles.append(article.encode('utf8'))
        _spUnicodeArticles.append(article)
    return _spArticles, _spUnicodeArticles


# Caches.
_ART_CACHE = {}
_SP_ART_CACHE = {}


def articlesDictsForLang(lang):
    """Return dictionaries of articles specific for the given language, or the
    default one if the language is not known."""
    if lang in _ART_CACHE:
        return _ART_CACHE[lang]
    artDicts = toDicts(LANG_ARTICLESget(lang, GENERIC_ARTICLES))
    _ART_CACHE[lang] = artDicts
    return artDicts


def spArticlesForLang(lang):
    """Return lists of articles (plus optional spaces) specific for the
    given language, or the default one if the language is not known."""
    if lang in _SP_ART_CACHE:
        return _SP_ART_CACHE[lang]
    spArticles = addTrailingSpace(LANG_ARTICLESget(lang, GENERIC_ARTICLES))
    _SP_ART_CACHE[lang] = spArticles
    return spArticles



import xml.etree.ElementTree as ET

def test_movie_xml(ia):
    movie = ia.get_movie('0133093')    # Matrix
    movie_xml = movie.asXML()
    movie_xml = movie_xml.encode('utf8', 'ignore')
    assert ET.fromstring(movie_xml) is not None



def test_references_to_titles_should_be_a_list(ia):
    person = ia.get_person('0000210', info=['biography'])   # Julia Roberts
    titles_refs = person.get_titlesRefs()
    assert 70 < len(titles_refs) < 100


def test_references_to_names_should_be_a_list(ia):
    person = ia.get_person('0000210', info=['biography'])  # Julia Roberts
    names_refs = person.get_namesRefs()
    assert 100 < len(names_refs) < 150



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
get_company.py

Usage: get_company "company_id"

Show some info about the company with the given company_id (e.g. '0071509'
for "Columbia Pictures [us]", using 'http' or 'mobile').
Notice that company_id, using 'sql', are not the same IDs used on the web.
"""

import sys

# Import the IMDbPY package.
try:
    import imdb
except ImportError:
    print('You need to install the IMDbPY package!')
    sys.exit(1)


if len(sys.argv) != 2:
    print('Only one argument is required:')
    print('  %s "company_id"' % sys.argv[0])
    sys.exit(2)

company_id = sys.argv[1]

i = imdb.IMDb()

try:
    # Get a company object with the data about the company identified by
    # the given company_id.
    company = i.get_company(company_id)
except imdb.IMDbError as e:
    print("Probably you're not connected to Internet.  Complete error report:")
    print(e)
    sys.exit(3)


if not company:
    print('It seems that there\'s no company with company_id "%s"' % company_id)
    sys.exit(4)

# XXX: this is the easier way to print the main info about a company;
# calling the summary() method of a company object will returns a string
# with the main information about the company.
# Obviously it's not really meaningful if you want to know how
# to access the data stored in a company object, so look below; the
# commented lines show some ways to retrieve information from a
# company object.
print(company.summary())



import re


def test_person_headshot_should_be_an_image_link(ia):
    person = ia.get_person('0000206', info=['biography'])   # Keanu Reeves
    assert re.match(r'^https?://.*\.jpg$', person['headshot'])


def test_person_full_size_headshot_should_be_an_image_link(ia):
    person = ia.get_person('0000206', info=['biography'])   # Keanu Reeves
    assert re.match(r'^https?://.*\.jpg$', person['full-size headshot'])


def test_person_headshot_if_none_should_be_excluded(ia):
    person = ia.get_person('0330139', info=['biography'])   # Deni Gordon
    assert 'headshot' not in person


def test_person_bio_is_present(ia):
    person = ia.get_person('0000206', info=['biography'])   # Keanu Reeves
    assert 'mini biography' in person


def test_person_birth_date_should_be_in_ymd_format(ia):
    person = ia.get_person('0000001', info=['biography'])   # Fred Astaire
    assert person.get('birth date') == '1899-05-10'


def test_person_birth_date_without_month_and_date_should_be_in_y00_format(ia):
    person = ia.get_person('0565883', info=['biography'])   # Belinda McClory
    assert person.get('birth date') == '1968-00-00'


def test_person_birth_date_without_itemprop_should_be_in_ymd_format(ia):
    person = ia.get_person('0000007', info=['biography'])   # Humphrey Bogart
    assert person.get('birth date') == '1899-12-25'


def test_person_birth_notes_should_contain_birth_place(ia):
    person = ia.get_person('0000001', info=['biography'])   # Fred Astaire
    assert person.get('birth notes') == 'Omaha, Nebraska, USA'


def test_person_death_date_should_be_in_ymd_format(ia):
    person = ia.get_person('0000001', info=['biography'])   # Fred Astaire
    assert person.get('death date') == '1987-06-22'


def test_person_death_date_without_itemprop_should_be_in_ymd_format(ia):
    person = ia.get_person('0000007', info=['biography'])   # Humphrey Bogart
    assert person.get('death date') == '1957-01-14'


def test_person_death_date_if_none_should_be_excluded(ia):
    person = ia.get_person('0000210', info=['biography'])   # Julia Roberts
    assert 'death date' not in person


def test_person_death_notes_should_contain_death_place_and_reason(ia):
    person = ia.get_person('0000001', info=['biography'])   # Fred Astaire
    assert person['death notes'] == 'in Los Angeles, California, USA (pneumonia)'


def test_person_death_notes_if_none_should_be_excluded(ia):
    person = ia.get_person('0000210', info=['biography'])   # Julia Roberts
    assert 'death notes' not in person


def test_person_birth_name_should_be_normalized(ia):
    data = ia.get_person('0000210', info=['biography'])     # Julia Roberts
    assert data.get('birth name') == 'Julia Fiona Roberts'


def test_person_nicknames_if_single_should_be_a_list_of_names(ia):
    person = ia.get_person('0000210', info=['biography'])   # Julia Roberts
    assert person.get('nick names') == ['Jules']


def test_person_nicknames_if_multiple_should_be_a_list_of_names(ia):
    person = ia.get_person('0000206', info=['biography'])   # Keanu Reeves
    assert person.get('nick names') == ['The Wall', 'The One']


def test_person_height_should_be_in_inches_and_meters(ia):
    person = ia.get_person('0000210', info=['biography'])   # Julia Roberts
    assert person.get('height') == '5\' 8" (1.73 m)'


def test_person_height_if_none_should_be_excluded(ia):
    person = ia.get_person('0617588', info=['biography'])   # Georges Melies
    assert 'height' not in person


def test_person_spouse_should_be_a_list(ia):
    person = ia.get_person('0000210', info=['biography'])   # Julia Roberts
    spouses = person.get('spouse', [])
    assert len(spouses) == 2


def test_person_trade_mark_should_be_a_list(ia):
    person = ia.get_person('0000210', info=['biography'])   # Julia Roberts
    trade_mark = person.get('trade mark', [])
    assert len(trade_mark) == 3


def test_person_trivia_should_be_a_list(ia):
    person = ia.get_person('0000210', info=['biography'])   # Julia Roberts
    trivia = person.get('trivia', [])
    assert len(trivia) > 90


def test_person_quotes_should_be_a_list(ia):
    person = ia.get_person('0000210', info=['biography'])   # Julia Roberts
    quotes = person.get('quotes', [])
    assert len(quotes) > 30


def test_person_salary_history_should_be_a_list(ia):
    person = ia.get_person('0000210', info=['biography'])   # Julia Roberts
    salary = person.get('salary history', [])
    assert len(salary) > 25



# Copyright 2005-2017 Davide Alberani <da@erlug.linux.it>
#                2006 Giuseppe "Cowo" Corbelli <cowo --> lugbs.linux.it>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""
This module provides the schema used to describe the layout of the database
used by the :mod:`imdb.parser.sql` package; functions to create/drop
tables and indexes are also provided.
"""

from __future__ import absolute_import, division, print_function, unicode_literals

from imdb.parser.sql.logging import logger


_dbschema_logger = logger.getChild('dbschema')


# Placeholders for column types.
INTCOL = 1
UNICODECOL = 2
STRINGCOL = 3
_strMap = {1: 'INTCOL', 2: 'UNICODECOL', 3: 'STRINGCOL'}


class DBCol(object):

    """Define column objects."""

    def __init__(self, name, kind, **params):
        self.name = name
        self.kind = kind
        self.index = None
        self.indexLen = None
        # If not None, two notations are accepted: 'TableName'
        # and 'TableName.ColName'; in the first case, 'id' is assumed
        # as the name of the pointed column.
        self.foreignKey = None
        if 'index' in params:
            self.index = params['index']
            del params['index']
        if 'indexLen' in params:
            self.indexLen = params['indexLen']
            del params['indexLen']
        if 'foreignKey' in params:
            self.foreignKey = params['foreignKey']
            del params['foreignKey']
        self.params = params

    def __str__(self):
        """Class representation."""
        s = '<DBCol %s %s' % (self.name, _strMap[self.kind])
        if self.index:
            s += ' INDEX'
            if self.indexLen:
                s += '[:%d]' % self.indexLen
        if self.foreignKey:
            s += ' FOREIGN'
        if 'default' in self.params:
            val = self.params['default']
            if val is not None:
                val = '"%s"' % val
            s += ' DEFAULT=%s' % val
        for param in self.params:
            if param == 'default':
                continue
            s += ' %s' % param.upper()
        s += '>'
        return s

    def __repr__(self):
        """Class representation."""
        s = '<DBCol(name="%s", %s' % (self.name, _strMap[self.kind])
        if self.index:
            s += ', index="%s"' % self.index
        if self.indexLen:
            s += ', indexLen=%d' % self.indexLen
        if self.foreignKey:
            s += ', foreignKey="%s"' % self.foreignKey
        for param in self.params:
            val = self.params[param]
            if isinstance(val, str):
                val = '"%s"' % val
            s += ', %s=%s' % (param, val)
        s += ')>'
        return s


class DBTable(object):

    """Define table objects."""

    def __init__(self, name, *cols, **kwds):
        self.name = name
        self.cols = cols
        # Default values.
        self.values = kwds.get('values', {})

    def __str__(self):
        """Class representation."""
        return '<DBTable %s (%d cols, %d values)>' % (
            self.name, len(self.cols), sum([len(v) for v in list(self.values.values())])
        )

    def __repr__(self):
        """Class representation."""
        s = '<DBTable(name="%s"' % self.name
        col_s = ', '.join([repr(col).rstrip('>').lstrip('<') for col in self.cols])
        if col_s:
            s += ', %s' % col_s
        if self.values:
            s += ', values=%s' % self.values
        s += ')>'
        return s


# Default values to insert in some tables: {'column': (list, of, values, ...)}
kindTypeDefs = {
    'kind': (
        'movie', 'tv series', 'tv movie', 'video movie',
        'tv mini series', 'video game', 'episode', 'short', 'tv short'
    )
}

companyTypeDefs = {
    'kind': (
        'distributors', 'production companies',
        'special effects companies', 'miscellaneous companies'
    )
}

infoTypeDefs = {
    'info': (
        'runtimes', 'color info', 'genres', 'languages',
        'certificates', 'sound mix', 'tech info', 'countries', 'taglines',
        'keywords', 'alternate versions', 'crazy credits', 'goofs',
        'soundtrack', 'quotes', 'release dates', 'trivia', 'locations',
        'mini biography', 'birth notes', 'birth date', 'height',
        'death date', 'spouse', 'other works', 'birth name',
        'salary history', 'nick names', 'books', 'agent address',
        'biographical movies', 'portrayed in', 'where now', 'trade mark',
        'interviews', 'article', 'magazine cover photo', 'pictorial',
        'death notes', 'LD disc format', 'LD year', 'LD digital sound',
        'LD official retail price', 'LD frequency response', 'LD pressing plant',
        'LD length', 'LD language', 'LD review', 'LD spaciality', 'LD release date',
        'LD production country', 'LD contrast', 'LD color rendition',
        'LD picture format', 'LD video noise', 'LD video artifacts',
        'LD release country', 'LD sharpness', 'LD dynamic range',
        'LD audio noise', 'LD color information', 'LD group genre',
        'LD quality program', 'LD close captions-teletext-ld-g',
        'LD category', 'LD analog left', 'LD certification',
        'LD audio quality', 'LD video quality', 'LD aspect ratio',
        'LD analog right', 'LD additional information',
        'LD number of chapter stops', 'LD dialogue intellegibility',
        'LD disc size', 'LD master format', 'LD subtitles',
        'LD status of availablility', 'LD quality of source',
        'LD number of sides', 'LD video standard', 'LD supplement',
        'LD original title', 'LD sound encoding', 'LD number', 'LD label',
        'LD catalog number', 'LD laserdisc title', 'screenplay-teleplay',
        'novel', 'adaption', 'book', 'production process protocol',
        'printed media reviews', 'essays', 'other literature', 'mpaa',
        'plot', 'votes distribution', 'votes', 'rating',
        'production dates', 'copyright holder', 'filming dates', 'budget',
        'weekend gross', 'gross', 'opening weekend', 'rentals',
        'admissions', 'studios', 'top 250 rank', 'bottom 10 rank'
    )
}

compCastTypeDefs = {
    'kind': ('cast', 'crew', 'complete', 'complete+verified')
}

linkTypeDefs = {
    'link': (
        'follows', 'followed by', 'remake of', 'remade as',
        'references', 'referenced in', 'spoofs', 'spoofed in',
        'features', 'featured in', 'spin off from', 'spin off',
        'version of', 'similar to', 'edited into',
        'edited from', 'alternate language version of',
        'unknown link'
    )
}

roleTypeDefs = {
    'role': (
        'actor', 'actress', 'producer', 'writer',
        'cinematographer', 'composer', 'costume designer',
        'director', 'editor', 'miscellaneous crew',
        'production designer', 'guest'
    )
}

# Schema of tables in our database.
# XXX: Foreign keys can be used to create constrains between tables,
#      but they create indexes in the database, and this
#      means poor performances at insert-time.
DB_SCHEMA = [
    DBTable('Name',
            # namePcodeCf is the soundex of the name in the canonical format.
            # namePcodeNf is the soundex of the name in the normal format, if
            # different from namePcodeCf.
            # surnamePcode is the soundex of the surname, if different from the
            # other two values.

            # The 'id' column is simply skipped by SQLObject (it's a default);
            # the alternateID attribute here will be ignored by SQLAlchemy.
            DBCol('id', INTCOL, notNone=True, alternateID=True),
            DBCol('name', UNICODECOL, notNone=True, index='idx_name', indexLen=6),
            DBCol('imdbIndex', STRINGCOL, length=12, default=None),
            DBCol('imdbID', INTCOL, default=None, index='idx_imdb_id'),
            DBCol('gender', STRINGCOL, length=1, default=None, index='idx_gender'),
            DBCol('namePcodeCf', STRINGCOL, length=5, default=None, index='idx_pcodecf'),
            DBCol('namePcodeNf', STRINGCOL, length=5, default=None, index='idx_pcodenf'),
            DBCol('surnamePcode', STRINGCOL, length=5, default=None, index='idx_pcode'),
            DBCol('md5sum', STRINGCOL, length=32, default=None, index='idx_md5')),

    DBTable('CharName',
            # namePcodeNf is the soundex of the name in the normal format.
            # surnamePcode is the soundex of the surname, if different
            # from namePcodeNf.
            DBCol('id', INTCOL, notNone=True, alternateID=True),
            DBCol('name', UNICODECOL, notNone=True, index='idx_name', indexLen=6),
            DBCol('imdbIndex', STRINGCOL, length=12, default=None),
            DBCol('imdbID', INTCOL, default=None, index='idx_imdb_id'),
            DBCol('namePcodeNf', STRINGCOL, length=5, default=None, index='idx_pcodenf'),
            DBCol('surnamePcode', STRINGCOL, length=5, default=None, index='idx_pcode'),
            DBCol('md5sum', STRINGCOL, length=32, default=None, index='idx_md5')),

    DBTable('CompanyName',
            # namePcodeNf is the soundex of the name in the normal format.
            # namePcodeSf is the soundex of the name plus the country code.
            DBCol('id', INTCOL, notNone=True, alternateID=True),
            DBCol('name', UNICODECOL, notNone=True, index='idx_name', indexLen=6),
            DBCol('countryCode', STRINGCOL, length=255, default=None, index='idx_ccode'),
            DBCol('imdbID', INTCOL, default=None, index='idx_imdb_id'),
            DBCol('namePcodeNf', STRINGCOL, length=5, default=None, index='idx_pcodenf'),
            DBCol('namePcodeSf', STRINGCOL, length=5, default=None, index='idx_pcodesf'),
            DBCol('md5sum', STRINGCOL, length=32, default=None, index='idx_md5')),

    DBTable('KindType',
            DBCol('id', INTCOL, notNone=True, alternateID=True),
            DBCol('kind', STRINGCOL, length=15, default=None, alternateID=True),
            values=kindTypeDefs),

    DBTable('Title',
            DBCol('id', INTCOL, notNone=True, alternateID=True),
            DBCol('title', UNICODECOL, notNone=True, index='idx_title', indexLen=10),
            DBCol('imdbIndex', STRINGCOL, length=12, default=None),
            DBCol('kindID', INTCOL, notNone=True, index='idx_kindid'),
            DBCol('productionYear', INTCOL, default=None, index='idx_year'),
            DBCol('imdbID', INTCOL, default=None, index="idx_imdb_id"),
            DBCol('phoneticCode', STRINGCOL, length=5, default=None, index='idx_pcode'),
            DBCol('episodeOfID', INTCOL, default=None, index='idx_epof'),
            DBCol('seasonNr', INTCOL, default=None, index="idx_season_nr"),
            DBCol('episodeNr', INTCOL, default=None, index="idx_episode_nr"),
            # Maximum observed length is 44; 49 can store 5 comma-separated
            #  year-year pairs.
            DBCol('seriesYears', STRINGCOL, length=49, default=None),
            DBCol('md5sum', STRINGCOL, length=32, default=None, index='idx_md5')),

    DBTable('CompanyType',
            DBCol('id', INTCOL, notNone=True, alternateID=True),
            DBCol('kind', STRINGCOL, length=32, default=None, alternateID=True),
            values=companyTypeDefs),

    DBTable('AkaName',
            DBCol('id', INTCOL, notNone=True, alternateID=True),
            DBCol('personID', INTCOL, notNone=True, index='idx_person'),
            DBCol('name', UNICODECOL, notNone=True, index='idx_name', indexLen=6),
            DBCol('imdbIndex', STRINGCOL, length=12, default=None),
            DBCol('namePcodeCf', STRINGCOL, length=5, default=None, index='idx_pcodecf'),
            DBCol('namePcodeNf', STRINGCOL, length=5, default=None, index='idx_pcodenf'),
            DBCol('surnamePcode', STRINGCOL, length=5, default=None, index='idx_pcode'),
            DBCol('md5sum', STRINGCOL, length=32, default=None, index='idx_md5')),

    DBTable('AkaTitle',
            # XXX: It's safer to set notNone to False, here.
            #      alias for akas are stored completely in the AkaTitle table;
            #      this means that episodes will set also a "tv series" alias name.
            #      Reading the aka-title.list file it looks like there are
            #      episode titles with aliases to different titles for both
            #      the episode and the series title, while for just the series
            #      there are no aliases.
            #      E.g.:
            #      aka title                                 original title
            # "Series, The" (2005) {The Episode}  "Other Title" (2005) {Other Title}
            # But there is no:
            # "Series, The" (2005)                "Other Title" (2005)
            DBCol('id', INTCOL, notNone=True, alternateID=True),
            DBCol('movieID', INTCOL, notNone=True, index='idx_movieid'),
            DBCol('title', UNICODECOL, notNone=True, index='idx_title', indexLen=10),
            DBCol('imdbIndex', STRINGCOL, length=12, default=None),
            DBCol('kindID', INTCOL, notNone=True, index='idx_kindid'),
            DBCol('productionYear', INTCOL, default=None, index='idx_year'),
            DBCol('phoneticCode', STRINGCOL, length=5, default=None, index='idx_pcode'),
            DBCol('episodeOfID', INTCOL, default=None, index='idx_epof'),
            DBCol('seasonNr', INTCOL, default=None),
            DBCol('episodeNr', INTCOL, default=None),
            DBCol('note', UNICODECOL, default=None),
            DBCol('md5sum', STRINGCOL, length=32, default=None, index='idx_md5')),

    DBTable('RoleType',
            DBCol('id', INTCOL, notNone=True, alternateID=True),
            DBCol('role', STRINGCOL, length=32, notNone=True, alternateID=True),
            values=roleTypeDefs),

    DBTable('CastInfo',
            DBCol('id', INTCOL, notNone=True, alternateID=True),
            DBCol('personID', INTCOL, notNone=True, index='idx_pid'),
            DBCol('movieID', INTCOL, notNone=True, index='idx_mid'),
            DBCol('personRoleID', INTCOL, default=None, index='idx_cid'),
            DBCol('note', UNICODECOL, default=None),
            DBCol('nrOrder', INTCOL, default=None),
            DBCol('roleID', INTCOL, notNone=True, index='idx_rid')),

    DBTable('CompCastType',
            DBCol('id', INTCOL, notNone=True, alternateID=True),
            DBCol('kind', STRINGCOL, length=32, notNone=True, alternateID=True),
            values=compCastTypeDefs),

    DBTable('CompleteCast',
            DBCol('id', INTCOL, notNone=True, alternateID=True),
            DBCol('movieID', INTCOL, index='idx_mid'),
            DBCol('subjectID', INTCOL, notNone=True, index='idx_sid'),
            DBCol('statusID', INTCOL, notNone=True)),

    DBTable('InfoType',
            DBCol('id', INTCOL, notNone=True, alternateID=True),
            DBCol('info', STRINGCOL, length=32, notNone=True, alternateID=True),
            values=infoTypeDefs),

    DBTable('LinkType',
            DBCol('id', INTCOL, notNone=True, alternateID=True),
            DBCol('link', STRINGCOL, length=32, notNone=True, alternateID=True),
            values=linkTypeDefs),

    DBTable('Keyword',
            DBCol('id', INTCOL, notNone=True, alternateID=True),
            # XXX: can't use alternateID=True, because it would create
            #      a UNIQUE index; unfortunately (at least with a common
            #      collation like utf8_unicode_ci) MySQL will consider
            #      some different keywords identical - like
            #      "fiancée" and "fiancee".
            DBCol('keyword', UNICODECOL, notNone=True, index='idx_keyword', indexLen=5),
            DBCol('phoneticCode', STRINGCOL, length=5, default=None, index='idx_pcode')),

    DBTable('MovieKeyword',
            DBCol('id', INTCOL, notNone=True, alternateID=True),
            DBCol('movieID', INTCOL, notNone=True, index='idx_mid'),
            DBCol('keywordID', INTCOL, notNone=True, index='idx_keywordid')),

    DBTable('MovieLink',
            DBCol('id', INTCOL, notNone=True, alternateID=True),
            DBCol('movieID', INTCOL, notNone=True, index='idx_mid'),
            DBCol('linkedMovieID', INTCOL, notNone=True, index='idx_lmid'),
            DBCol('linkTypeID', INTCOL, notNone=True, index='idx_ltypeid')),

    DBTable('MovieInfo',
            DBCol('id', INTCOL, notNone=True, alternateID=True),
            DBCol('movieID', INTCOL, notNone=True, index='idx_mid'),
            DBCol('infoTypeID', INTCOL, notNone=True, index='idx_infotypeid'),
            DBCol('info', UNICODECOL, notNone=True, index='idx_info', indexLen=10),
            DBCol('note', UNICODECOL, default=None)),

    DBTable('MovieCompanies',
            DBCol('id', INTCOL, notNone=True, alternateID=True),
            DBCol('movieID', INTCOL, notNone=True, index='idx_mid'),
            DBCol('companyID', INTCOL, notNone=True, index='idx_cid'),
            DBCol('companyTypeID', INTCOL, notNone=True, index='idx_ctypeid'),
            DBCol('note', UNICODECOL, default=None)),

    DBTable('PersonInfo',
            DBCol('id', INTCOL, notNone=True, alternateID=True),
            DBCol('personID', INTCOL, notNone=True, index='idx_pid'),
            DBCol('infoTypeID', INTCOL, notNone=True, index='idx_itypeid'),
            DBCol('info', UNICODECOL, notNone=True),
            DBCol('note', UNICODECOL, default=None))
]


# Functions to manage tables.
def dropTables(tables, ifExists=True):
    """Drop the tables."""
    # In reverse order (useful to avoid errors about foreign keys).
    DB_TABLES_DROP = list(tables)
    DB_TABLES_DROP.reverse()
    for table in DB_TABLES_DROP:
        _dbschema_logger.info('dropping table %s', table._imdbpyName)
        table.dropTable(ifExists)


def createTables(tables, ifNotExists=True):
    """Create the tables and insert default values."""
    for table in tables:
        # Create the table.
        _dbschema_logger.info('creating table %s', table._imdbpyName)
        table.createTable(ifNotExists)
        # Insert default values, if any.
        if table._imdbpySchema.values:
            _dbschema_logger.info('inserting values into table %s', table._imdbpyName)
            for key in table._imdbpySchema.values:
                for value in table._imdbpySchema.values[key]:
                    table(**{key: str(value)})


def createIndexes(tables, ifNotExists=True):
    """Create the indexes in the database.
    Return a list of errors, if any."""
    errors = []
    for table in tables:
        _dbschema_logger.info('creating indexes for table %s', table._imdbpyName)
        try:
            table.addIndexes(ifNotExists)
        except Exception as e:
            errors.append(e)
            continue
    return errors



def test_movie_localized_title(ia):
    movie = ia.get_movie('2991224', info=['main'])
    title = movie.get('localized title', '')
    assert title == 'Tangerines - Mandarini'


def test_movie_original_title(ia):
    movie = ia.get_movie('2991224', info=['main'])
    title = movie.get('original title', '')
    assert title == 'Mandariinid'


def test_movie_title(ia):
    movie = ia.get_movie('2991224', info=['main'])
    title = movie.get('title', '')
    assert title == 'Tangerines'




# Copyright 2009 H. Turgut Uyar <uyar@tekir.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""
This package provides scripts and files for internationalization of IMDbPY.
"""

from __future__ import absolute_import, division, print_function, unicode_literals

import gettext
import os

LOCALE_DIR = os.path.dirname(__file__)

gettext.bindtextdomain('imdbpy', LOCALE_DIR)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
get_movie.py

Usage: get_movie "movie_id"

Show some info about the movie with the given movie_id (e.g. '0133093'
for "The Matrix", using 'http' or 'mobile').
Notice that movie_id, using 'sql', are not the same IDs used on the web.
"""

import sys

# Import the IMDbPY package.
try:
    import imdb
except ImportError:
    print('You need to install the IMDbPY package!')
    sys.exit(1)


if len(sys.argv) != 2:
    print('Only one argument is required:')
    print('  %s "movie_id"' % sys.argv[0])
    sys.exit(2)

movie_id = sys.argv[1]

i = imdb.IMDb()

try:
    # Get a Movie object with the data about the movie identified by
    # the given movie_id.
    movie = i.get_movie(movie_id)
except imdb.IMDbError as e:
    print("Probably you're not connected to Internet.  Complete error report:")
    print(e)
    sys.exit(3)


if not movie:
    print('It seems that there\'s no movie with movie_id "%s"' % movie_id)
    sys.exit(4)

# XXX: this is the easier way to print the main info about a movie;
# calling the summary() method of a Movie object will returns a string
# with the main information about the movie.
# Obviously it's not really meaningful if you want to know how
# to access the data stored in a Movie object, so look below; the
# commented lines show some ways to retrieve information from a
# Movie object.
print(movie.summary())

# Show some info about the movie.
# This is only a short example; you can get a longer summary using
# 'print movie.summary()' and the complete set of information looking for
# the output of the movie.keys() method.
#
# print '==== "%s" / movie_id: %s ====' % (movie['title'], movie_id)
# XXX: use the IMDb instance to get the IMDb web URL for the movie.
# imdbURL = i.get_imdbURL(movie)
# if imdbURL:
#    print 'IMDb URL: %s' % imdbURL
#
# XXX: many keys return a list of values, like "genres".
# genres = movie.get('genres')
# if genres:
#    print 'Genres: %s' % ' '.join(genres)
#
# XXX: even when only one value is present (e.g.: movie with only one
#      director), fields that can be multiple are ALWAYS a list.
#      Note that the 'name' variable is a Person object, but since its
#      __str__() method returns a string with the name, we can use it
#      directly, instead of name['name']
# director = movie.get('director')
# if director:
#    print 'Director(s): ',
#    for name in director:
#        sys.stdout.write('%s ' % name)
#    print ''
#
# XXX: notice that every name in the cast is a Person object, with a
#      currentRole instance variable, which is a string for the played role.
# cast = movie.get('cast')
# if cast:
#    print 'Cast: '
#    cast = cast[:5]
#    for name in cast:
#        print '      %s (%s)' % (name['name'], name.currentRole)
# XXX: some information are not lists of strings or Person objects, but simple
#      strings, like 'rating'.
# rating = movie.get('rating')
# if rating:
#    print 'Rating: %s' % rating
# XXX: an example of how to use information sets; retrieve the "trivia"
#      info set; check if it contains some data, select and print a
#      random entry.
# import random
# i.update(movie, info=['trivia'])
# trivia = movie.get('trivia')
# if trivia:
#    rand_trivia = trivia[random.randrange(len(trivia))]
#    print 'Random trivia: %s' % rand_trivia




def test_movie_full_credits(ia):
    movie = ia.get_movie('0133093', info=['full credits']) # Matrix
    assert 'cast' in movie
    assert len(movie['cast']) == 41

def test_movie_full_credits_for_tv_show(ia):
    movie = ia.get_movie('0098904', info=['full credits']) # Seinfeld
    assert 'cast' in movie
    assert len(movie['cast']) > 1300 and len(movie['cast']) < 1350

def test_movie_full_credits_contains_headshot(ia):
    movie = ia.get_movie('0133093', info=['main', 'full credits'])      # Matrix
    assert 'headshot' in movie['cast'][0] # Keanu Reeves
    assert 'nopicture' not in movie['cast'][0]['headshot'] # is real headshot, not default



def test_search_keyword_check_list_of_keywords(ia):
    keywords = ia.search_keyword('zoolander')
    assert 'reference-to-zoolander' in keywords


def test_search_keyword_if_multiple_should_list_correct_number_of_keywords(ia):
    keywords = ia.search_keyword('messiah')
    assert 50 <= len(keywords) <= 80


def test_search_keyword_if_too_many_should_list_upper_limit_of_keywords(ia):
    keywords = ia.search_keyword('computer')
    assert len(keywords) == 200


def test_search_keyword_if_none_result_should_be_empty(ia):
    keywords = ia.search_keyword('%e3%82%a2')
    assert keywords == []


def test_get_keyword_pagination(ia):
    superheroes_without_page_param = ia.get_keyword('superhero')
    superheroes_page_one = ia.get_keyword('superhero', page=1)
    superheroes_page_two = ia.get_keyword('superhero', page=2)
    for i in range(50):
        assert superheroes_without_page_param[i]['title'] == superheroes_page_one[i]['title']
        assert superheroes_without_page_param[i]['title'] != superheroes_page_two[i]['title']

    



def test_movie_official_sites_should_be_a_list(ia):
    movie = ia.get_movie('0133093', info=['official sites'])    # Matrix
    official_sites = movie.get('official sites', [])
    assert len(official_sites) == 3


def test_movie_official_sites_if_none_should_be_excluded(ia):
    movie = ia.get_movie('1863157', info=['official sites'])    # Ates Parcasi
    assert 'official sites' not in movie


def test_movie_sound_clips_should_be_a_list(ia):
    movie = ia.get_movie('0133093', info=['official sites'])    # Matrix
    sound_clips = movie.get('sound clips', [])
    assert len(sound_clips) == 3


def test_movie_sound_clips_if_none_should_be_excluded(ia):
    movie = ia.get_movie('1863157', info=['official sites'])    # Ates Parcasi
    assert 'sound clips' not in movie



def test_person_in_movie(ia):
    movie = ia.get_movie('0133093', info=['main'])  # Matrix
    person = ia.get_person('0000206', info=['main'])  # Keanu Reeves
    assert person in movie


def test_key_in_movie(ia):
    movie = ia.get_movie('0133093', info=['main'])  # Matrix
    assert 'cast' in movie


def test_movie_in_person(ia):
    movie = ia.get_movie('0133093', info=['main'])  # Matrix
    person = ia.get_person('0000206', info=['main'])  # Keanu Reeves
    assert movie in person


def test_key_in_person(ia):
    person = ia.get_person('0000206')  # Keanu Reeves
    assert 'filmography' in person


def test_key_in_company(ia):
    company = ia.get_company('0017902', info=['main'])  # Pixar
    assert 'name' in company




def test_movie_parental_guide_contains_mpaa_rating(ia):
    movie = ia.get_movie('0133093', info=['parents guide'])  # Matrix
    assert movie.get('mpaa') == "Rated R for sci-fi violence and brief language"



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
get_first_company.py

Usage: get_first_company "company name"

Search for the given name and print the best matching result.
"""

import sys

# Import the IMDbPY package.
try:
    import imdb
except ImportError:
    print('You need to install the IMDbPY package!')
    sys.exit(1)


if len(sys.argv) != 2:
    print('Only one argument is required:')
    print('  %s "company name"' % sys.argv[0])
    sys.exit(2)

name = sys.argv[1]


i = imdb.IMDb()

try:
    # Do the search, and get the results (a list of company objects).
    results = i.search_company(name)
except imdb.IMDbError as e:
    print("Probably you're not connected to Internet.  Complete error report:")
    print(e)
    sys.exit(3)

if not results:
    print('No matches for "%s", sorry.' % name)
    sys.exit(0)

# Print only the first result.
print('    Best match for "%s"' % name)

# This is a company instance.
company = results[0]

# So far the company object only contains basic information like the
# name; retrieve main information:
i.update(company)

print(company.summary())



# Copyright 2009-2020 Davide Alberani <da@erlug.linux.it>
#                2018 H. Turgut Uyar <uyar@tekir.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""
This module provides the classes (and the instances) that are used to parse
the pages for the lists of top 250 and bottom 100 movies.

Pages:

http://www.imdb.com/chart/top

http://www.imdb.com/chart/bottom
"""

from __future__ import absolute_import, division, print_function, unicode_literals

from imdb.utils import analyze_title

from .piculet import Path, Rule, Rules, reducers
from .utils import DOMParserBase, analyze_imdbid


class DOMHTMLTop250Parser(DOMParserBase):
    """A parser for the "top 250 movies" page."""
    ranktext = 'top 250 rank'

    rules = [
        Rule(
            key='chart',
            extractor=Rules(
                foreach='//tbody[@class="lister-list"]/tr',
                rules=[
                    Rule(
                        key='rank',
                        extractor=Path('.//span[@name="rk"]/@data-value',
                                       reduce=reducers.first,
                                       transform=int)
                    ),
                    Rule(
                        key='rating',
                        extractor=Path('.//span[@name="ir"]/@data-value',
                                       reduce=reducers.first,
                                       transform=lambda x: round(float(x), 1))
                    ),
                    Rule(
                        key='movieID',
                        extractor=Path('./td[@class="titleColumn"]/a/@href', reduce=reducers.first)
                    ),
                    Rule(
                        key='title',
                        extractor=Path('./td[@class="titleColumn"]/a/text()')
                    ),
                    Rule(
                        key='year',
                        extractor=Path('./td[@class="titleColumn"]/span/text()')
                    ),
                    Rule(
                        key='votes',
                        extractor=Path('.//span[@name="nv"]/@data-value', reduce=reducers.first,
                                       transform=int)
                    )
                ]
            )
        )
    ]

    def postprocess_data(self, data):
        if (not data) or ('chart' not in data):
            return []

        movies = []
        for entry in data['chart']:
            if ('movieID' not in entry) or ('rank' not in entry) or ('title' not in entry):
                continue

            movie_id = analyze_imdbid(entry['movieID'])
            if movie_id is None:
                continue
            del entry['movieID']

            entry[self.ranktext] = entry['rank']
            del entry['rank']

            title = analyze_title(entry['title'] + ' ' + entry.get('year', ''))
            entry.update(title)

            movies.append((movie_id, entry))
        return movies


class DOMHTMLBottom100Parser(DOMHTMLTop250Parser):
    """A parser for the "bottom 100 movies" page."""
    ranktext = 'bottom 100 rank'


class DOMHTMLMoviemeter100Parser(DOMHTMLTop250Parser):
    """A parser for the "Most Popular Movies" page."""
    ranktext = 'popular movies 100 rank'


class DOMHTMLTVmeter100Parser(DOMHTMLTop250Parser):
    """A parser for the "Most Popular TV Shows" page."""
    ranktext = 'popular tv 100 rank'


class DOMHTMLTVTop250Parser(DOMHTMLTop250Parser):
    """A parser for the "Top Rated TV Shows" page."""
    ranktext = 'top tv 250 rank'


class DOMHTMLTopIndian250Parser(DOMHTMLTop250Parser):
    """A parser for the "Top Rated Indian Movies" page."""
    ranktext = 'top indian 250 rank'


_OBJECTS = {
    'top250_parser': ((DOMHTMLTop250Parser,), None),
    'bottom100_parser': ((DOMHTMLBottom100Parser,), None),
    'moviemeter100_parser': ((DOMHTMLMoviemeter100Parser,), None),
    'toptv250_parser': ((DOMHTMLTVTop250Parser,), None),
    'tvmeter100_parser': ((DOMHTMLTVmeter100Parser,), None),
    'topindian250_parser': ((DOMHTMLTopIndian250Parser,), None)
}



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
search_person.py

Usage: search_person "person name"

Search for the given name and print the results.
"""

import sys

# Import the IMDbPY package.
try:
    import imdb
except ImportError:
    print('You bad boy!  You need to install the IMDbPY package!')
    sys.exit(1)


if len(sys.argv) != 2:
    print('Only one argument is required:')
    print('  %s "person name"' % sys.argv[0])
    sys.exit(2)

name = sys.argv[1]


i = imdb.IMDb()

out_encoding = sys.stdout.encoding or sys.getdefaultencoding()

try:
    # Do the search, and get the results (a list of Person objects).
    results = i.search_person(name)
except imdb.IMDbError as e:
    print("Probably you're not connected to Internet.  Complete error report:")
    print(e)
    sys.exit(3)

# Print the results.
print('    %s result%s for "%s":' % (len(results),
                                     ('', 's')[len(results) != 1],
                                     name))
print('personID\t: imdbID : name')

# Print the long imdb name for every person.
for person in results:
    outp = '%s\t: %s : %s' % (person.personID, i.get_imdbID(person),
                               person['long imdb name'])
    print(outp)



import os
import sys

import setuptools


# version of the software from imdb/version.py
exec(compile(open('imdb/version.py').read(), 'imdb/version.py', 'exec'))

home_page = 'https://imdbpy.github.io/'

long_desc = """IMDbPY is a Python package useful to retrieve and
manage the data of the IMDb movie database about movies, people,
characters and companies.

Platform-independent and written in Python 3
it can retrieve data from both the IMDb's web server and a local copy
of the whole database.

IMDbPY package can be very easily used by programmers and developers
to provide access to the IMDb's data to their programs.

Some simple example scripts - useful for the end users - are included
in this package; other IMDbPY-based programs are available at the
home page: %s
""" % home_page

dwnl_url = 'https://imdbpy.github.io/downloads/'

classifiers = """\
Development Status :: 5 - Production/Stable
Environment :: Console
Environment :: Web Environment
Intended Audience :: Developers
Intended Audience :: End Users/Desktop
License :: OSI Approved :: GNU General Public License (GPL)
Natural Language :: English
Natural Language :: Italian
Natural Language :: Turkish
Programming Language :: Python
Programming Language :: Python :: 3.9
Programming Language :: Python :: 3.8
Programming Language :: Python :: 3.7
Programming Language :: Python :: 3.6
Programming Language :: Python :: 3.5
Programming Language :: Python :: 2.7
Programming Language :: Python :: Implementation :: CPython
Programming Language :: Python :: Implementation :: PyPy
Operating System :: OS Independent
Topic :: Database :: Front-Ends
Topic :: Internet :: WWW/HTTP :: Dynamic Content :: CGI Tools/Libraries
Topic :: Software Development :: Libraries :: Python Modules
"""

keywords = ['imdb', 'movie', 'people', 'database', 'cinema', 'film', 'person',
            'cast', 'actor', 'actress', 'director', 'sql', 'character',
            'company', 'package', 'plain text data files',
            'keywords', 'top250', 'bottom100', 'xml']

scripts = [
    './bin/get_first_movie.py',
    './bin/imdbpy2sql.py',
    './bin/s32imdbpy.py',
    './bin/get_movie.py',
    './bin/search_movie.py',
    './bin/get_first_person.py',
    './bin/get_person.py',
    './bin/search_person.py',
    './bin/get_company.py',
    './bin/search_company.py',
    './bin/get_first_company.py',
    './bin/get_keyword.py',
    './bin/search_keyword.py',
    './bin/get_top_bottom_movies.py'
]

params = {
    # Meta-information.
    'name': 'IMDbPY',
    'version': __version__,
    'description': 'Python package to access the IMDb\'s database',
    'long_description': long_desc,
    'author': 'Davide Alberani',
    'author_email': 'da@erlug.linux.it',
    'contact': 'IMDbPY-devel mailing list',
    'contact_email': 'imdbpy-devel@lists.sourceforge.net',
    'maintainer': 'Davide Alberani',
    'maintainer_email': 'da@erlug.linux.it',
    'license': 'GPL',
    'platforms': 'any',
    'keywords': keywords,
    'classifiers': [_f for _f in classifiers.split("\n") if _f],
    'url': home_page,
    'download_url': dwnl_url,
    'scripts': scripts,
    'package_data': {
        # Here, the "*" represents any possible language ID.
        'imdb.locale': [
            'imdbpy.pot',
            'imdbpy-*.po',
            '*/LC_MESSAGES/imdbpy.mo',
        ],
    },
    'install_requires': ['SQLAlchemy', 'lxml'],
    'extras_require': {
        'dev': [
            'flake8',
            'flake8-isort',
            'readme_renderer'
        ],
        'doc': [
            'sphinx',
            'sphinx_rtd_theme'
        ],
        'test': [
            'pytest',
            'pytest-cov',
            'pytest-profiling'
        ]
    },
    'packages': setuptools.find_packages(),
    'entry_points': """
        [console_scripts]
        imdbpy=imdb.cli:main
    """
}


ERR_MSG = """
====================================================================
  ERROR
  =====

  Aaargh!  An error!  An error!
  Curse my metal body, I wasn't fast enough.  It's all my fault!

  Anyway, if you were trying to build a package or install IMDbPY to your
  system, looks like we're unable to fetch or install some dependencies.

  The best solution is to resolve these dependencies (maybe you're
  not connected to Internet?)

  The caught exception, is re-raise below:
"""


REBUILDMO_DIR = os.path.join('imdb', 'locale')
REBUILDMO_NAME = 'rebuildmo'


def runRebuildmo():
    """Call the function to rebuild the locales."""
    cwd = os.getcwd()
    path = list(sys.path)
    languages = []
    try:
        import importlib
        scriptPath = os.path.dirname(__file__)
        modulePath = os.path.join(cwd, scriptPath, REBUILDMO_DIR)
        sys.path += [modulePath, '.', cwd]
        rebuildmo = importlib.import_module(os.path.join(REBUILDMO_DIR, REBUILDMO_NAME).replace(os.path.sep, '.'))
        os.chdir(modulePath)
        languages = rebuildmo.rebuildmo()
        print('Created locale for: %s.' % ' '.join(languages))
    except Exception as e:
        print('ERROR: unable to rebuild .mo files; caught exception %s' % e)
    sys.path = path
    os.chdir(cwd)
    return languages


def hasCommand():
    """Return true if at least one command is found on the command line."""
    args = sys.argv[1:]
    if '--help' in args:
        return False
    if '-h' in args:
        return False
    for arg in args:
        if arg and not arg.startswith('-'):
            return True
    return False


try:
    if hasCommand():
        runRebuildmo()
except SystemExit:
    print(ERR_MSG)

setuptools.setup(**params)



def test_top_tv_should_contain_100_entries(ia):
    chart = ia.get_top250_tv()
    assert len(chart) == 250


def test_top_tv_entries_should_have_rank(ia):
    movies = ia.get_top250_tv()
    for rank, movie in enumerate(movies):
        assert movie['top tv 250 rank'] == rank + 1


def test_top_tv_entries_should_have_movie_id(ia):
    movies = ia.get_top250_tv()
    for movie in movies:
        assert movie.movieID.isdigit()


def test_top_tv_entries_should_have_title(ia):
    movies = ia.get_top250_tv()
    for movie in movies:
        assert 'title' in movie


def test_top_tv_entries_should_be_movies(ia):
    movies = ia.get_top250_tv()
    for movie in movies:
        assert movie['kind'] == 'movie'


def test_top_tv_entries_should_have_year(ia):
    movies = ia.get_top250_tv()
    for movie in movies:
        assert isinstance(movie['year'], int)


def test_top_tv_entries_should_have_high_ratings(ia):
    movies = ia.get_top250_tv()
    for movie in movies:
        assert movie['rating'] > 8.0


def test_top_tv_entries_should_have_minimal_number_of_votes(ia):
    movies = ia.get_top250_tv()
    for movie in movies:
        assert movie['votes'] >= 1500  # limit stated by IMDb



# Copyright 2009 H. Turgut Uyar <uyar@tekir.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""
This script builds the .mo files, from the .po files.
"""

import glob
import os

import msgfmt


def rebuildmo():
    lang_glob = 'imdbpy-*.po'
    created = []
    for input_file in sorted(glob.glob(lang_glob)):
        lang = input_file[7:-3]
        if not os.path.exists(lang):
            os.mkdir(lang)
        mo_dir = os.path.join(lang, 'LC_MESSAGES')
        if not os.path.exists(mo_dir):
            os.mkdir(mo_dir)
        output_file = os.path.join(mo_dir, 'imdbpy.mo')
        msgfmt.make(input_file, output_file)
        created.append(lang)
    return created


if __name__ == '__main__':
    languages = rebuildmo()
    print('Created locale for: %s.' % ' '.join(languages))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
s32imdbpy.py script.

This script imports the s3 dataset distributed by IMDb into a SQL database.

Copyright 2017-2018 Davide Alberani <da@erlug.linux.it>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
"""

import os
import glob
import gzip
import logging
import argparse
import sqlalchemy

try:
    from tqdm import tqdm
    HAS_TQDM = True
except ImportError:
    HAS_TQDM = False

from imdb.parser.s3.utils import DB_TRANSFORM, title_soundex, name_soundexes

TSV_EXT = '.tsv.gz'
# how many entries to write to the database at a time.
BLOCK_SIZE = 10000

logger = logging.getLogger()
logger.setLevel(logging.INFO)
metadata = sqlalchemy.MetaData()


def generate_content(fd, headers, table):
    """Generate blocks of rows to be written to the database.

    :param fd: a file descriptor for the .tsv.gz file
    :type fd: :class:`_io.TextIOWrapper`
    :param headers: headers in the file
    :type headers: list
    :param table: the table that will populated
    :type table: :class:`sqlalchemy.Table`
    :returns: block of data to insert
    :rtype: list
    """
    data = []
    headers_len = len(headers)
    data_transf = {}
    table_name = table.name
    for column, conf in DB_TRANSFORM.get(table_name, {}).items():
        if 'transform' in conf:
            data_transf[column] = conf['transform']
    for line in fd:
        s_line = line.decode('utf-8').strip().split('\t')
        if len(s_line) != headers_len:
            continue
        info = dict(zip(headers, [x if x != r'\N' else None for x in s_line]))
        for key, tranf in data_transf.items():
            if key not in info:
                continue
            info[key] = tranf(info[key])
        if table_name == 'title_basics':
            info['t_soundex'] = title_soundex(info['primaryTitle'])
        elif table_name == 'title_akas':
            info['t_soundex'] = title_soundex(info['title'])
        elif table_name == 'name_basics':
            info['ns_soundex'], info['sn_soundex'], info['s_soundex'] = name_soundexes(info['primaryName'])
        data.append(info)
        if len(data) >= BLOCK_SIZE:
            yield data
            data = []
    if data:
        yield data
        data = []


def build_table(fn, headers):
    """Build a Table object from a .tsv.gz file.

    :param fn: the .tsv.gz file
    :type fn: str
    :param headers: headers in the file
    :type headers: list
    """
    logging.debug('building table for file %s' % fn)
    table_name = fn.replace(TSV_EXT, '').replace('.', '_')
    table_map = DB_TRANSFORM.get(table_name) or {}
    columns = []
    all_headers = set(headers)
    all_headers.update(table_map.keys())
    for header in all_headers:
        col_info = table_map.get(header) or {}
        col_type = col_info.get('type') or sqlalchemy.UnicodeText
        if 'length' in col_info and col_type is sqlalchemy.String:
            col_type = sqlalchemy.String(length=col_info['length'])
        col_args = {
            'name': header,
            'type_': col_type,
            'index': col_info.get('index', False)
        }
        col_obj = sqlalchemy.Column(**col_args)
        columns.append(col_obj)
    return sqlalchemy.Table(table_name, metadata, *columns)


def import_file(fn, engine):
    """Import data from a .tsv.gz file.

    :param fn: the .tsv.gz file
    :type fn: str
    :param engine: SQLAlchemy engine
    :type engine: :class:`sqlalchemy.engine.base.Engine`
    """
    logging.info('begin processing file %s' % fn)
    connection = engine.connect()
    count = 0
    nr_of_lines = 0
    fn_basename = os.path.basename(fn)
    with gzip.GzipFile(fn, 'rb') as gz_file:
        gz_file.readline()
        for line in gz_file:
            nr_of_lines += 1
    with gzip.GzipFile(fn, 'rb') as gz_file:
        headers = gz_file.readline().decode('utf-8').strip().split('\t')
        logging.debug('headers of file %s: %s' % (fn, ','.join(headers)))
        table = build_table(fn_basename, headers)
        try:
            table.drop()
            logging.debug('table %s dropped' % table.name)
        except:
            pass
        insert = table.insert()
        metadata.create_all(tables=[table])
        if HAS_TQDM and logger.isEnabledFor(logging.DEBUG):
            tqdm_ = tqdm
        else:
            tqdm_ = lambda it, **kwargs: it
        try:
            for block in generate_content(tqdm_(gz_file, total=nr_of_lines), headers, table):
                try:
                    connection.execute(insert, block)
                except Exception as e:
                    logging.error('error processing data: %d entries lost: %s' % (len(block), e))
                    continue
                count += len(block)
                percent = count * 100 / nr_of_lines
        except Exception as e:
            logging.error('error processing data on table %s: %s' % (table.name, e))
        logging.info('processed file %s: %d entries' % (fn, count))


def import_dir(dir_name, engine):
    """Import data from a series of .tsv.gz files.

    :param dir_name: directory containing the .tsv.gz files
    :type dir_name: str
    :param engine: SQLAlchemy engine
    :type engine: :class:`sqlalchemy.engine.base.Engine`
    """
    for fn in glob.glob(os.path.join(dir_name, '*%s' % TSV_EXT)):
        if not os.path.isfile(fn):
            logging.debug('skipping file %s' % fn)
            continue
        import_file(fn, engine)


if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('tsv_files_dir')
    parser.add_argument('db_uri')
    parser.add_argument('--verbose', help='increase verbosity and show progress', action='store_true')
    args = parser.parse_args()
    dir_name = args.tsv_files_dir
    db_uri = args.db_uri
    if args.verbose:
        logger.setLevel(logging.DEBUG)
    engine = sqlalchemy.create_engine(db_uri, encoding='utf-8', echo=False)
    metadata.bind = engine
    import_dir(dir_name, engine)




def test_movie_keywords_should_be_a_list_of_keywords(ia):
    movie = ia.get_movie('0133093', info=['keywords'])  # Matrix
    keywords = movie.get('keywords', [])
    assert 250 <= len(keywords) <= 400
    assert {'computer-hacker', 'messiah', 'artificial-reality'}.issubset(set(keywords))


def test_movie_keywords_if_none_should_be_excluded(ia):
    movie = ia.get_movie('1863157', info=['keywords'])  # Ates Parcasi
    assert 'keywords' not in movie
